@@ -16,6 +16,13 @@ import { fetching } from './node/fetch';
 
 const CLIENT_ID = '01ab8ac9400c4e429b23';
 const GITHUB_TOKEN_URL = 'https://vscode.dev/codeExchangeProxyEndpoints/github/login/oauth/access_token';
+
+// This is the error message that we throw if the login was cancelled for any reason. Extensions
+// calling `getSession` can handle this error to know that the user cancelled the login.
+const CANCELLATION_ERROR = 'Cancelled';
+// These error messages are internal and should not be shown to the user in any way.
+const TIMED_OUT_ERROR = 'Timed out';
+const USER_CANCELLATION_ERROR = 'User Cancelled';
 const NETWORK_ERROR = 'network error';
 
 const REDIRECT_URL_STABLE = 'https://vscode.dev/redirect';
@@ -24,7 +31,7 @@ const REDIRECT_URL_INSIDERS = 'https://insiders.vscode.dev/redirect';
 export interface IGitHubServer {
 	login(scopes: string): Promise<string>;
 	getUserInfo(token: string): Promise<{ id: string; accountName: string }>;
-	sendAdditionalTelemetryInfo(token: string): Promise<void>;
+	sendAdditionalTelemetryInfo(session: vscode.AuthenticationSession): Promise<void>;
 	friendlyName: string;
 }
 
@@ -41,7 +48,7 @@ async function getScopes(token: string, serverUri: vscode.Uri, logger: Log): Pro
 		const result = await fetching(serverUri.toString(), {
 			headers: {
 				Authorization: `token ${token}`,
-				'User-Agent': 'Visual-Studio-Code'
+				'User-Agent': `${vscode.env.appName} (${vscode.env.appHost})`
 			}
 		});
 
@@ -122,17 +129,17 @@ export class GitHubServer implements IGitHubServer {
 		let userCancelled: boolean | undefined;
 		const yes = vscode.l10n.t('Yes');
 		const no = vscode.l10n.t('No');
-		const promptToContinue = async () => {
+		const promptToContinue = async (mode: string) => {
 			if (userCancelled === undefined) {
 				// We haven't had a failure yet so wait to prompt
 				return;
 			}
 			const message = userCancelled
-				? vscode.l10n.t('Having trouble logging in? Would you like to try a different way?')
-				: vscode.l10n.t('You have not yet finished authorizing this extension to use GitHub. Would you like to keep trying?');
+				? vscode.l10n.t('Having trouble logging in? Would you like to try a different way? ({0})', mode)
+				: vscode.l10n.t('You have not yet finished authorizing this extension to use GitHub. Would you like to try a different way? ({0})', mode);
 			const result = await vscode.window.showWarningMessage(message, yes, no);
 			if (result !== yes) {
-				throw new Error('Cancelled');
+				throw new Error(CANCELLATION_ERROR);
 			}
 		};
 
@@ -146,7 +153,7 @@ export class GitHubServer implements IGitHubServer {
 				return await this.doLoginWithoutLocalServer(scopes, nonce, callbackUri);
 			} catch (e) {
 				this._logger.error(e);
-				userCancelled = e.message ?? e === 'User Cancelled';
+				userCancelled = e.message ?? e === USER_CANCELLATION_ERROR;
 			}
 		}
 
@@ -160,22 +167,20 @@ export class GitHubServer implements IGitHubServer {
 			supportedTarget
 		) {
 			try {
-				await promptToContinue();
+				await promptToContinue(vscode.l10n.t('local server'));
 				return await this.doLoginWithLocalServer(scopes);
 			} catch (e) {
-				this._logger.error(e);
-				userCancelled = e.message ?? e === 'User Cancelled';
+				userCancelled = this.processLoginError(e);
 			}
 		}
 
 		// We only can use the Device Code flow when we have a full node environment because of CORS.
 		if (typeof navigator === 'undefined') {
 			try {
-				await promptToContinue();
+				await promptToContinue(vscode.l10n.t('device code'));
 				return await this.doLoginDeviceCodeFlow(scopes);
 			} catch (e) {
-				this._logger.error(e);
-				userCancelled = e.message ?? e === 'User Cancelled';
+				userCancelled = this.processLoginError(e);
 			}
 		}
 
@@ -183,15 +188,14 @@ export class GitHubServer implements IGitHubServer {
 		// With that said, GitHub Enterprise isn't used by Settings Sync so we can use PATs for that.
 		if (!supportedClient || this._type === AuthProviderType.githubEnterprise) {
 			try {
-				await promptToContinue();
+				await promptToContinue(vscode.l10n.t('personal access token'));
 				return await this.doLoginWithPat(scopes);
 			} catch (e) {
-				this._logger.error(e);
-				userCancelled = e.message ?? e === 'User Cancelled';
+				userCancelled = this.processLoginError(e);
 			}
 		}
 
-		throw new Error(userCancelled ? 'Cancelled' : 'No auth flow succeeded.');
+		throw new Error(userCancelled ? CANCELLATION_ERROR : 'No auth flow succeeded.');
 	}
 
 	private async doLoginWithoutLocalServer(scopes: string, nonce: string, callbackUri: vscode.Uri): Promise<string> {
@@ -232,8 +236,8 @@ export class GitHubServer implements IGitHubServer {
 			try {
 				return await Promise.race([
 					codeExchangePromise.promise,
-					new Promise<string>((_, reject) => setTimeout(() => reject('Timed out'), 300_000)), // 5min timeout
-					promiseFromEvent<any, any>(token.onCancellationRequested, (_, __, reject) => { reject('User Cancelled'); }).promise
+					new Promise<string>((_, reject) => setTimeout(() => reject(TIMED_OUT_ERROR), 300_000)), // 5min timeout
+					promiseFromEvent<any, any>(token.onCancellationRequested, (_, __, reject) => { reject(USER_CANCELLATION_ERROR); }).promise
 				]);
 			} finally {
 				this._pendingNonces.delete(scopes);
@@ -273,8 +277,8 @@ export class GitHubServer implements IGitHubServer {
 				vscode.env.openExternal(vscode.Uri.parse(`http://127.0.0.1:${port}/signin?nonce=${encodeURIComponent(server.nonce)}`));
 				const { code } = await Promise.race([
 					server.waitForOAuthResponse(),
-					new Promise<any>((_, reject) => setTimeout(() => reject('Timed out'), 300_000)), // 5min timeout
-					promiseFromEvent<any, any>(token.onCancellationRequested, (_, __, reject) => { reject('User Cancelled'); }).promise
+					new Promise<any>((_, reject) => setTimeout(() => reject(TIMED_OUT_ERROR), 300_000)), // 5min timeout
+					promiseFromEvent<any, any>(token.onCancellationRequested, (_, __, reject) => { reject(USER_CANCELLATION_ERROR); }).promise
 				]);
 				codeToExchange = code;
 			} finally {
@@ -317,7 +321,7 @@ export class GitHubServer implements IGitHubServer {
 			}, button);
 
 		if (modalResult !== button) {
-			throw new Error('User Cancelled');
+			throw new Error(USER_CANCELLATION_ERROR);
 		}
 
 		await vscode.env.clipboard.writeText(json.user_code);
@@ -340,14 +344,14 @@ export class GitHubServer implements IGitHubServer {
 			}, button);
 
 		if (modalResult !== button) {
-			throw new Error('User Cancelled');
+			throw new Error(USER_CANCELLATION_ERROR);
 		}
 
 		const description = `${vscode.env.appName} (${scopes})`;
 		const uriToOpen = await vscode.env.asExternalUri(this.baseUri.with({ path: '/settings/tokens/new', query: `description=${description}&scopes=${scopes.split(' ').join(',')}` }));
 		await vscode.env.openExternal(uriToOpen);
 		const token = await vscode.window.showInputBox({ placeHolder: `ghp_1a2b3c4...`, prompt: `GitHub Personal Access Token - ${scopes}`, ignoreFocusOut: true });
-		if (!token) { throw new Error('User Cancelled'); }
+		if (!token) { throw new Error(USER_CANCELLATION_ERROR); }
 
 		const tokenScopes = await getScopes(token, this.getServerUri('/'), this._logger); // Example: ['repo', 'user']
 		const scopesList = scopes.split(' '); // Example: 'read:user repo user:email'
@@ -392,7 +396,7 @@ export class GitHubServer implements IGitHubServer {
 			for (let i = 0; i < attempts; i++) {
 				await new Promise(resolve => setTimeout(resolve, json.interval * 1000));
 				if (token.isCancellationRequested) {
-					throw new Error('User Cancelled');
+					throw new Error(USER_CANCELLATION_ERROR);
 				}
 				let accessTokenResult;
 				try {
@@ -423,7 +427,7 @@ export class GitHubServer implements IGitHubServer {
 				return accessTokenJson.access_token;
 			}
 
-			throw new Error('Cancelled');
+			throw new Error(TIMED_OUT_ERROR);
 		});
 	}
 
@@ -506,7 +510,7 @@ export class GitHubServer implements IGitHubServer {
 			result = await fetching(this.getServerUri('/user').toString(), {
 				headers: {
 					Authorization: `token ${token}`,
-					'User-Agent': 'Visual-Studio-Code'
+					'User-Agent': `${vscode.env.appName} (${vscode.env.appHost})`
 				}
 			});
 		} catch (ex) {
@@ -539,7 +543,7 @@ export class GitHubServer implements IGitHubServer {
 		}
 	}
 
-	public async sendAdditionalTelemetryInfo(token: string): Promise<void> {
+	public async sendAdditionalTelemetryInfo(session: vscode.AuthenticationSession): Promise<void> {
 		if (!vscode.env.isTelemetryEnabled) {
 			return;
 		}
@@ -550,22 +554,22 @@ export class GitHubServer implements IGitHubServer {
 		}
 
 		if (this._type === AuthProviderType.github) {
-			return await this.checkUserDetails(token);
+			return await this.checkUserDetails(session);
 		}
 
 		// GHES
-		await this.checkEnterpriseVersion(token);
+		await this.checkEnterpriseVersion(session.accessToken);
 	}
 
-	private async checkUserDetails(token: string): Promise<void> {
+	private async checkUserDetails(session: vscode.AuthenticationSession): Promise<void> {
 		let edu: string | undefined;
 
 		try {
 			const result = await fetching('https://education.github.com/api/user', {
 				headers: {
-					Authorization: `token ${token}`,
+					Authorization: `token ${session.accessToken}`,
 					'faculty-check-preview': 'true',
-					'User-Agent': 'Visual-Studio-Code'
+					'User-Agent': `${vscode.env.appName} (${vscode.env.appHost})`
 				}
 			});
 
@@ -576,22 +580,11 @@ export class GitHubServer implements IGitHubServer {
 					: json.faculty
 						? 'faculty'
 						: 'none';
+			} else {
+				edu = 'unknown';
 			}
 		} catch (e) {
-			// No-op
-		}
-
-		let managed: string | undefined;
-		try {
-			const user = await this.getUserInfo(token);
-			// Apparently, this is how you tell if a user is an EMU...
-			managed = user.accountName.includes('_') ? 'true' : 'false';
-		} catch (e) {
-			// No-op
-		}
-
-		if (edu === undefined && managed === undefined) {
-			return;
+			edu = 'unknown';
 		}
 
 		/* __GDPR__
@@ -602,8 +595,9 @@ export class GitHubServer implements IGitHubServer {
 			}
 		*/
 		this._telemetryReporter.sendTelemetryEvent('session', {
-			isEdu: edu ?? 'unknown',
-			isManaged: managed ?? 'unknown'
+			isEdu: edu,
+			// Apparently, this is how you tell if a user is an EMU...
+			isManaged: session.account.label.includes('_') ? 'true' : 'false'
 		});
 	}
 
@@ -614,7 +608,7 @@ export class GitHubServer implements IGitHubServer {
 				const result = await fetching(this.getServerUri('/meta').toString(), {
 					headers: {
 						Authorization: `token ${token}`,
-						'User-Agent': 'Visual-Studio-Code'
+						'User-Agent': `${vscode.env.appName} (${vscode.env.appHost})`
 					}
 				});
 
@@ -641,4 +635,12 @@ export class GitHubServer implements IGitHubServer {
 			// No-op
 		}
 	}
+
+	private processLoginError(error: Error): boolean {
+		if (error.message === CANCELLATION_ERROR) {
+			throw error;
+		}
+		this._logger.error(error.message ?? error);
+		return error.message === USER_CANCELLATION_ERROR;
+	}
 }
