@@ -235,26 +235,16 @@ function terminateWhenParentTerminates() {
 	}
 }
 
-// TODO@bpasero remove this when sandbox is final
 function configureCrashReporter() {
-	const crashReporterSandboxedHint = process.env['VSCODE_CRASH_REPORTER_SANDBOXED_HINT'];
-	if (crashReporterSandboxedHint) {
-		addCrashReporterParameter('_sandboxed', 'true');
-	}
-
 	const crashReporterProcessType = process.env['VSCODE_CRASH_REPORTER_PROCESS_TYPE'];
 	if (crashReporterProcessType) {
-		addCrashReporterParameter('processType', crashReporterProcessType);
-	}
-}
-
-function addCrashReporterParameter(key, value) {
-	try {
-		if (process['crashReporter'] && typeof process['crashReporter'].addExtraParameter === 'function' /* Electron only */) {
-			process['crashReporter'].addExtraParameter(key, value);
+		try {
+			if (process['crashReporter'] && typeof process['crashReporter'].addExtraParameter === 'function' /* Electron only */) {
+				process['crashReporter'].addExtraParameter('processType', crashReporterProcessType);
+			}
+		} catch (error) {
+			console.error(error);
 		}
-	} catch (error) {
-		console.error(error);
 	}
 }
 
