@@ -62,7 +62,8 @@ export interface IProductConfiguration {
 
 	readonly nameShort: string;
 	readonly nameLong: string;
-	readonly nameLongSubtitle?: string;
+	readonly welcomePageTitle?: string;
+	readonly welcomePageSubtitle?: string;
 
 	readonly win32AppUserModelId?: string;
 	readonly win32MutexName?: string;
@@ -90,7 +91,6 @@ export interface IProductConfiguration {
 	readonly tasConfig?: {
 		endpoint: string;
 		telemetryEventName: string;
-		featuresTelemetryPropertyName: string;
 		assignmentContextTelemetryPropertyName: string;
 	};
 
@@ -192,12 +192,7 @@ export interface IProductConfiguration {
 
 	readonly 'editSessions.store'?: Omit<ConfigurationSyncStore, 'insidersUrl' | 'stableUrl'>;
 	readonly darwinUniversalAssetId?: string;
-	readonly profileTemplates?: IProfileTemplateInfo[];
-}
-
-export interface IProfileTemplateInfo {
-	readonly name: string;
-	readonly url: string;
+	readonly profileTemplatesUrl?: string;
 }
 
 export interface ITunnelApplicationConfig {
@@ -259,14 +254,24 @@ export interface IRemoteExtensionTip {
 	friendlyName: string;
 	extensionId: string;
 	supportedPlatforms?: PlatformName[];
-	showInStartEntry?: boolean;
+	startEntry?: {
+		helpLink: string;
+		startConnectLabel: string;
+		startCommand: string;
+		priority: number;
+	};
 }
 
 export interface IVirtualWorkspaceExtensionTip {
 	friendlyName: string;
 	extensionId: string;
 	supportedPlatforms?: PlatformName[];
-	showInStartEntry?: boolean;
+	startEntry: {
+		helpLink: string;
+		startConnectLabel: string;
+		startCommand: string;
+		priority: number;
+	};
 }
 
 export interface ISurveyData {
