@@ -7,7 +7,7 @@ import { transformErrorForSerialization } from 'vs/base/common/errors';
 import { Emitter, Event } from 'vs/base/common/event';
 import { Disposable, IDisposable } from 'vs/base/common/lifecycle';
 import { getAllMethodNames } from 'vs/base/common/objects';
-import { globals, isWeb } from 'vs/base/common/platform';
+import { isWeb } from 'vs/base/common/platform';
 import * as strings from 'vs/base/common/strings';
 
 const INITIALIZE = '$initialize';
@@ -136,12 +136,12 @@ class SimpleWorkerProtocol {
 	public listen(eventName: string, arg: any): Event<any> {
 		let req: string | null = null;
 		const emitter = new Emitter<any>({
-			onFirstListenerAdd: () => {
+			onWillAddFirstListener: () => {
 				req = String(++this._lastSentReq);
 				this._pendingEmitters.set(req, emitter);
 				this._send(new SubscribeEventMessage(this._workerId, req, eventName, arg));
 			},
-			onLastListenerRemove: () => {
+			onDidRemoveLastListener: () => {
 				this._pendingEmitters.delete(req!);
 				this._send(new UnsubscribeEventMessage(this._workerId, req!));
 				req = null;
@@ -324,12 +324,14 @@ export class SimpleWorkerClient<W extends object, H extends object> extends Disp
 
 		// Gather loader configuration
 		let loaderConfiguration: any = null;
-		if (typeof globals.require !== 'undefined' && typeof globals.require.getConfig === 'function') {
+
+		const globalRequire: { getConfig?(): object } | undefined = (globalThis as any).require;
+		if (typeof globalRequire !== 'undefined' && typeof globalRequire.getConfig === 'function') {
 			// Get the configuration from the Monaco AMD Loader
-			loaderConfiguration = globals.require.getConfig();
-		} else if (typeof globals.requirejs !== 'undefined') {
+			loaderConfiguration = globalRequire.getConfig();
+		} else if (typeof (globalThis as any).requirejs !== 'undefined') {
 			// Get the configuration from requirejs
-			loaderConfiguration = globals.requirejs.s.contexts._.config;
+			loaderConfiguration = (globalThis as any).requirejs.s.contexts._.config;
 		}
 
 		const hostMethods = getAllMethodNames(host);
@@ -527,17 +529,17 @@ export class SimpleWorkerServer<H extends object> {
 
 			// Since this is in a web worker, enable catching errors
 			loaderConfig.catchError = true;
-			globals.require.config(loaderConfig);
+			globalThis.require.config(loaderConfig);
 		}
 
 		return new Promise<string[]>((resolve, reject) => {
 			// Use the global require to be sure to get the global config
 
 			// ESM-comment-begin
-			const req = (globals.require || require);
+			const req = (globalThis.require || require);
 			// ESM-comment-end
 			// ESM-uncomment-begin
-			// const req = globals.require;
+			// const req = globalThis.require;
 			// ESM-uncomment-end
 
 			req([moduleId], (module: { create: IRequestHandlerFactory<H> }) => {
