@@ -48,14 +48,11 @@ export function listProcesses(rootPid: number): Promise<ProcessItem> {
 
 		function findName(cmd: string): string {
 
-			const SHARED_PROCESS_HINT = /--vscode-window-kind=shared-process/;
-			const ISSUE_REPORTER_HINT = /--vscode-window-kind=issue-reporter/;
-			const PROCESS_EXPLORER_HINT = /--vscode-window-kind=process-explorer/;
-			const UTILITY_NETWORK_HINT = /--utility-sub-type=network/;
-			const UTILITY_EXTENSION_HOST_HINT = /--utility-sub-type=node.mojom.NodeService/;
-			const WINDOWS_CRASH_REPORTER = /--crashes-directory/;
-			const WINDOWS_PTY = /\\pipe\\winpty-control/;
-			const WINDOWS_CONSOLE_HOST = /conhost\.exe/;
+			const UTILITY_NETWORK_HINT = /--utility-sub-type=network/i;
+			const NODEJS_PROCESS_HINT = /--ms-enable-electron-run-as-node/i;
+			const WINDOWS_CRASH_REPORTER = /--crashes-directory/i;
+			const WINDOWS_PTY = /\\pipe\\winpty-control/i;
+			const WINDOWS_CONSOLE_HOST = /conhost\.exe/i;
 			const TYPE = /--type=([a-zA-Z-]+)/;
 
 			// find windows crash reporter
@@ -68,7 +65,7 @@ export function listProcesses(rootPid: number): Promise<ProcessItem> {
 				return 'winpty-process';
 			}
 
-			//find windows console host process
+			// find windows console host process
 			if (WINDOWS_CONSOLE_HOST.exec(cmd)) {
 				return 'console-window-host (Windows internal process)';
 			}
@@ -77,27 +74,15 @@ export function listProcesses(rootPid: number): Promise<ProcessItem> {
 			let matches = TYPE.exec(cmd);
 			if (matches && matches.length === 2) {
 				if (matches[1] === 'renderer') {
-					if (SHARED_PROCESS_HINT.exec(cmd)) {
-						return 'shared-process';
-					}
-
-					if (ISSUE_REPORTER_HINT.exec(cmd)) {
-						return 'issue-reporter';
-					}
-
-					if (PROCESS_EXPLORER_HINT.exec(cmd)) {
-						return 'process-explorer';
-					}
-
 					return `window`;
 				} else if (matches[1] === 'utility') {
 					if (UTILITY_NETWORK_HINT.exec(cmd)) {
 						return 'utility-network-service';
 					}
 
-					if (UTILITY_EXTENSION_HOST_HINT.exec(cmd)) {
-						return 'extension-host';
-					}
+					return 'utility-process';
+				} else if (matches[1] === 'extensionHost') {
+					return 'extension-host'; // normalize remote extension host type
 				}
 				return matches[1];
 			}
@@ -114,9 +99,15 @@ export function listProcesses(rootPid: number): Promise<ProcessItem> {
 
 			if (result) {
 				if (cmd.indexOf('node ') < 0 && cmd.indexOf('node.exe') < 0) {
-					return `electron_node ${result}`;
+					return `electron-nodejs (${result})`;
 				}
 			}
+
+			// find Electron node.js processes
+			if (NODEJS_PROCESS_HINT.exec(cmd)) {
+				return `electron-nodejs (${cmd})`;
+			}
+
 			return cmd;
 		}
 
@@ -136,7 +127,7 @@ export function listProcesses(rootPid: number): Promise<ProcessItem> {
 				}
 			};
 
-			(import('windows-process-tree')).then(windowsProcessTree => {
+			(import('@vscode/windows-process-tree')).then(windowsProcessTree => {
 				windowsProcessTree.getProcessList(rootPid, (processList) => {
 					if (!processList) {
 						reject(new Error(`Root process ${rootPid} not found`));
@@ -178,7 +169,10 @@ export function listProcesses(rootPid: number): Promise<ProcessItem> {
 							reject(new Error(`Root process ${rootPid} not found`));
 						}
 					});
-				}, windowsProcessTree.ProcessDataFlag.CommandLine | windowsProcessTree.ProcessDataFlag.Memory);
+				},
+					// Workaround duplicate enum identifiers issue in @vscode/windows-process-tree
+					// Ref https://github.com/microsoft/vscode/pull/179508
+					(windowsProcessTree.ProcessDataFlag as any).CommandLine | (windowsProcessTree.ProcessDataFlag as any).Memory);
 			});
 		} else {	// OS X & Linux
 			function calculateLinuxCpuUsage() {
