@@ -810,13 +810,13 @@ function createKeyboardNavigationEventFilter(keybindingService: IKeybindingServi
 
 		const result = keybindingService.softDispatch(event, event.target);
 
-		if (result?.kind === ResultKind.MoreChordsNeeded) {
+		if (result.kind === ResultKind.MoreChordsNeeded) {
 			inMultiChord = true;
 			return false;
 		}
 
 		inMultiChord = false;
-		return !result;
+		return result.kind === ResultKind.NoMatchingKb;
 	};
 }
 
