@@ -212,6 +212,11 @@ export interface IQuickInput extends IDisposable {
 
 	description: string | undefined;
 
+	/**
+	 * Should be an HTMLElement (TODO: move this entire file into browser)
+	 */
+	widget: any | undefined;
+
 	step: number | undefined;
 
 	totalSteps: number | undefined;
