@@ -5,27 +5,31 @@
 
 import { createDecorator } from 'vs/platform/instantiation/common/instantiation';
 import { Event } from 'vs/base/common/event';
+import { localize } from 'vs/nls';
 
-export interface IRemoteTunnelAccount {
+export interface IRemoteTunnelSession {
 	readonly providerId: string;
-	readonly token: string;
+	readonly sessionId: string;
 	readonly accountLabel: string;
+	readonly token?: string;
 }
 
 export const IRemoteTunnelService = createDecorator<IRemoteTunnelService>('IRemoteTunnelService');
 export interface IRemoteTunnelService {
 	readonly _serviceBrand: undefined;
 
-	readonly onDidTokenFailed: Event<boolean>;
-
 	readonly onDidChangeTunnelStatus: Event<TunnelStatus>;
 	getTunnelStatus(): Promise<TunnelStatus>;
 
-	getAccount(): Promise<IRemoteTunnelAccount | undefined>;
-	readonly onDidChangeAccount: Event<IRemoteTunnelAccount | undefined>;
-	updateAccount(account: IRemoteTunnelAccount | undefined): Promise<TunnelStatus>;
+	getSession(): Promise<IRemoteTunnelSession | undefined>;
+	readonly onDidChangeSession: Event<IRemoteTunnelSession | undefined>;
 
-	getHostName(): Promise<string | undefined>;
+	readonly onDidTokenFailed: Event<IRemoteTunnelSession | undefined>;
+	initialize(session: IRemoteTunnelSession | undefined): Promise<TunnelStatus>;
+
+	startTunnel(session: IRemoteTunnelSession): Promise<TunnelStatus>;
+	stopTunnel(): Promise<void>;
+	getTunnelName(): Promise<string | undefined>;
 
 }
 
@@ -45,24 +49,25 @@ export namespace TunnelStates {
 	}
 	export interface Disconnected {
 		readonly type: 'disconnected';
+		readonly onTokenFailed?: IRemoteTunnelSession;
 	}
-
-	export const disconnected: Disconnected = { type: 'disconnected' };
-	export const uninitialized: Uninitialized = { type: 'uninitialized' };
+	export const disconnected = (onTokenFailed?: IRemoteTunnelSession): Disconnected => ({ type: 'disconnected', onTokenFailed });
 	export const connected = (info: ConnectionInfo): Connected => ({ type: 'connected', info });
 	export const connecting = (progress?: string): Connecting => ({ type: 'connecting', progress });
+	export const uninitialized: Uninitialized = { type: 'uninitialized' };
 
 }
 
 export interface ConnectionInfo {
 	link: string;
 	domain: string;
-	hostName: string;
+	tunnelName: string;
+	isAttached: boolean;
 }
 
 export const CONFIGURATION_KEY_PREFIX = 'remote.tunnels.access';
 export const CONFIGURATION_KEY_HOST_NAME = CONFIGURATION_KEY_PREFIX + '.hostNameOverride';
+export const CONFIGURATION_KEY_PREVENT_SLEEP = CONFIGURATION_KEY_PREFIX + '.preventSleep';
 
-export const LOG_FILE_NAME = 'remoteTunnelService.log';
-export const LOGGER_NAME = 'remoteTunnelService';
-export const LOG_CHANNEL_ID = 'remoteTunnelServiceLog';
+export const LOG_ID = 'remoteTunnelService';
+export const LOGGER_NAME = localize('remoteTunnelLog', "Remote Tunnel Service");
