@@ -3,9 +3,9 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { IEnvironmentService } from 'vs/platform/environment/common/environment';
+import { localize } from 'vs/nls';
 import { AbstractLogger, ILogger, ILoggerService } from 'vs/platform/log/common/log';
-import { IUserDataSyncLogService } from 'vs/platform/userDataSync/common/userDataSync';
+import { IUserDataSyncLogService, USER_DATA_SYNC_LOG_ID } from 'vs/platform/userDataSync/common/userDataSync';
 
 export class UserDataSyncLogService extends AbstractLogger implements IUserDataSyncLogService {
 
@@ -14,10 +14,9 @@ export class UserDataSyncLogService extends AbstractLogger implements IUserDataS
 
 	constructor(
 		@ILoggerService loggerService: ILoggerService,
-		@IEnvironmentService environmentService: IEnvironmentService
 	) {
 		super();
-		this.logger = this._register(loggerService.createLogger(environmentService.userDataSyncLogResource, { name: 'settingssync' }));
+		this.logger = this._register(loggerService.createLogger(USER_DATA_SYNC_LOG_ID, { name: localize('userDataSyncLog', "Settings Sync") }));
 	}
 
 	trace(message: string, ...args: any[]): void {
