@@ -42,13 +42,6 @@ import { IPolicyService } from 'vs/platform/policy/common/policy';
 import { IUserDataProfile } from 'vs/platform/userDataProfile/common/userDataProfile';
 import { IStateService } from 'vs/platform/state/node/state';
 import { IUserDataProfilesMainService } from 'vs/platform/userDataProfile/electron-main/userDataProfile';
-import { INativeHostMainService } from 'vs/platform/native/electron-main/nativeHostMainService';
-import { OneDataSystemAppender } from 'vs/platform/telemetry/node/1dsAppender';
-import { ITelemetryServiceConfig, TelemetryService } from 'vs/platform/telemetry/common/telemetryService';
-import { getPiiPathsFromEnvironment, isInternalTelemetry, ITelemetryAppender, supportsTelemetry } from 'vs/platform/telemetry/common/telemetryUtils';
-import { resolveCommonProperties } from 'vs/platform/telemetry/common/commonProperties';
-import { hostname, release } from 'os';
-import { resolveMachineId } from 'vs/platform/telemetry/electron-main/telemetryUtils';
 import { ILoggerMainService } from 'vs/platform/log/electron-main/loggerService';
 import { firstOrDefault } from 'vs/base/common/arrays';
 
@@ -93,8 +86,6 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 
 	private static readonly windowControlHeightStateStorageKey = 'windowControlHeight';
 
-	private static sandboxState: boolean | undefined = undefined;
-
 	//#region Events
 
 	private readonly _onWillLoad = this._register(new Emitter<ILoadEvent>());
@@ -126,9 +117,6 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 	private _lastFocusTime = -1;
 	get lastFocusTime(): number { return this._lastFocusTime; }
 
-	private _isSandboxed = false;
-	get isSandboxed(): boolean { return this._isSandboxed; }
-
 	get backupPath(): string | undefined { return this._config?.backupPath; }
 
 	get openedWorkspace(): IWorkspaceIdentifier | ISingleFolderWorkspaceIdentifier | undefined { return this._config?.workspace; }
@@ -204,13 +192,11 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 		@IProductService private readonly productService: IProductService,
 		@IProtocolMainService private readonly protocolMainService: IProtocolMainService,
 		@IWindowsMainService private readonly windowsMainService: IWindowsMainService,
-		@IStateService private readonly stateService: IStateService,
-		@INativeHostMainService private readonly nativeHostMainService: INativeHostMainService
+		@IStateService private readonly stateService: IStateService
 	) {
 		super();
 
 		//#region create browser window
-		let useSandbox = false;
 		{
 			// Load window state
 			const [state, hasMultipleDisplays] = this.restoreWindowState(config.state);
@@ -221,24 +207,8 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 			// after the call to maximize/fullscreen (see below)
 			const isFullscreenOrMaximized = (this.windowState.mode === WindowMode.Maximized || this.windowState.mode === WindowMode.Fullscreen);
 
-			if (typeof CodeWindow.sandboxState === 'undefined') {
-				// we should only check this once so that we do not end up
-				// with some windows in sandbox mode and some not!
-				CodeWindow.sandboxState = this.stateService.getItem<boolean>('window.experimental.useSandbox', false);
-			}
-
 			const windowSettings = this.configurationService.getValue<IWindowSettings | undefined>('window');
 
-			if (typeof windowSettings?.experimental?.useSandbox === 'boolean') {
-				useSandbox = windowSettings.experimental.useSandbox;
-			} else if (this.productService.quality === 'stable' && CodeWindow.sandboxState) {
-				useSandbox = true;
-			} else {
-				useSandbox = this.productService.quality !== 'stable';
-			}
-
-			this._isSandboxed = useSandbox;
-
 			const options: BrowserWindowConstructorOptions & { experimentalDarkMode: boolean } = {
 				width: this.windowState.width,
 				height: this.windowState.height,
@@ -260,18 +230,7 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 					// Enable experimental css highlight api https://chromestatus.com/feature/5436441440026624
 					// Refs https://github.com/microsoft/vscode/issues/140098
 					enableBlinkFeatures: 'HighlightAPI',
-					...useSandbox ?
-
-						// Sandbox
-						{
-							sandbox: true
-						} :
-
-						// No Sandbox
-						{
-							nodeIntegration: true,
-							contextIsolation: false
-						}
+					sandbox: true
 				},
 				experimentalDarkMode: true
 			};
@@ -447,7 +406,7 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 		this.createTouchBar();
 
 		// Eventing
-		this.registerListeners(useSandbox);
+		this.registerListeners();
 	}
 
 	setRepresentedFilename(filename: string): void {
@@ -551,12 +510,12 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 		});
 	}
 
-	private registerListeners(sandboxed: boolean): void {
+	private registerListeners(): void {
 
 		// Window error conditions to handle
-		this._win.on('unresponsive', () => this.onWindowError(WindowError.UNRESPONSIVE, { sandboxed }));
-		this._win.webContents.on('render-process-gone', (event, details) => this.onWindowError(WindowError.PROCESS_GONE, { ...details, sandboxed }));
-		this._win.webContents.on('did-fail-load', (event, exitCode, reason) => this.onWindowError(WindowError.LOAD, { reason, exitCode, sandboxed }));
+		this._win.on('unresponsive', () => this.onWindowError(WindowError.UNRESPONSIVE));
+		this._win.webContents.on('render-process-gone', (event, details) => this.onWindowError(WindowError.PROCESS_GONE, { ...details }));
+		this._win.webContents.on('did-fail-load', (event, exitCode, reason) => this.onWindowError(WindowError.LOAD, { reason, exitCode }));
 
 		// Prevent windows/iframes from blocking the unload
 		// through DOM events. We have our own logic for
@@ -653,10 +612,10 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 		return this.marketplaceHeadersPromise;
 	}
 
-	private async onWindowError(error: WindowError.UNRESPONSIVE, details: { sandboxed: boolean }): Promise<void>;
-	private async onWindowError(error: WindowError.PROCESS_GONE, details: { reason: string; exitCode: number; sandboxed: boolean }): Promise<void>;
-	private async onWindowError(error: WindowError.LOAD, details: { reason: string; exitCode: number; sandboxed: boolean }): Promise<void>;
-	private async onWindowError(type: WindowError, details: { reason?: string; exitCode?: number; sandboxed: boolean }): Promise<void> {
+	private async onWindowError(error: WindowError.UNRESPONSIVE): Promise<void>;
+	private async onWindowError(error: WindowError.PROCESS_GONE, details: { reason: string; exitCode: number }): Promise<void>;
+	private async onWindowError(error: WindowError.LOAD, details: { reason: string; exitCode: number }): Promise<void>;
+	private async onWindowError(type: WindowError, details?: { reason?: string; exitCode?: number }): Promise<void> {
 
 		switch (type) {
 			case WindowError.PROCESS_GONE:
@@ -674,7 +633,6 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 		type WindowErrorClassification = {
 			type: { classification: 'SystemMetaData'; purpose: 'PerformanceAndHealth'; isMeasurement: true; comment: 'The type of window error to understand the nature of the error better.' };
 			reason: { classification: 'SystemMetaData'; purpose: 'PerformanceAndHealth'; comment: 'The reason of the window error to understand the nature of the error better.' };
-			sandboxed: { classification: 'SystemMetaData'; purpose: 'PerformanceAndHealth'; comment: 'If the window was sandboxed or not.' };
 			code: { classification: 'SystemMetaData'; purpose: 'PerformanceAndHealth'; isMeasurement: true; comment: 'The exit code of the window process to understand the nature of the error better' };
 			owner: 'bpasero';
 			comment: 'Provides insight into reasons the vscode window had an error.';
@@ -683,13 +641,11 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 			type: WindowError;
 			reason: string | undefined;
 			code: number | undefined;
-			sandboxed: string;
 		};
 		this.telemetryService.publicLog2<WindowErrorEvent, WindowErrorClassification>('windowerror', {
 			type,
 			reason: details?.reason,
-			code: details?.exitCode,
-			sandboxed: details?.sandboxed ? '1' : '0'
+			code: details?.exitCode
 		});
 
 		// Inform User if non-recoverable
@@ -748,103 +704,35 @@ export class CodeWindow extends Disposable implements ICodeWindow {
 
 				// Process gone
 				else if (type === WindowError.PROCESS_GONE) {
-
-					// Windows: running as admin with AppLocker enabled is unsupported
-					//          when sandbox: true.
-					//          we cannot detect AppLocker use currently, but make a
-					//          guess based on the reason and exit code.
-					if (isWindows && details?.reason === 'launch-failed' && details.exitCode === 18 && await this.nativeHostMainService.isAdmin(undefined)) {
-						await this.handleWindowsAdminCrash(details);
+					let message: string;
+					if (!details) {
+						message = localize('appGone', "The window terminated unexpectedly");
+					} else {
+						message = localize('appGoneDetails', "The window terminated unexpectedly (reason: '{0}', code: '{1}')", details.reason, details.exitCode ?? '<unknown>');
 					}
 
-					// Any other crash: offer to restart
-					else {
-						let message: string;
-						if (!details) {
-							message = localize('appGone', "The window terminated unexpectedly");
-						} else {
-							message = localize('appGoneDetails', "The window terminated unexpectedly (reason: '{0}', code: '{1}')", details.reason, details.exitCode ?? '<unknown>');
-						}
+					// Show Dialog
+					const { response, checkboxChecked } = await this.dialogMainService.showMessageBox({
+						type: 'warning',
+						buttons: [
+							this._config?.workspace ? localize({ key: 'reopen', comment: ['&& denotes a mnemonic'] }, "&&Reopen") : localize({ key: 'newWindow', comment: ['&& denotes a mnemonic'] }, "&&New Window"),
+							localize({ key: 'close', comment: ['&& denotes a mnemonic'] }, "&&Close")
+						],
+						message,
+						detail: this._config?.workspace ?
+							localize('appGoneDetailWorkspace', "We are sorry for the inconvenience. You can reopen the window to continue where you left off.") :
+							localize('appGoneDetailEmptyWindow', "We are sorry for the inconvenience. You can open a new empty window to start again."),
+						checkboxLabel: this._config?.workspace ? localize('doNotRestoreEditors', "Don't restore editors") : undefined
+					}, this._win);
 
-						// Show Dialog
-						const { response, checkboxChecked } = await this.dialogMainService.showMessageBox({
-							type: 'warning',
-							buttons: [
-								this._config?.workspace ? localize({ key: 'reopen', comment: ['&& denotes a mnemonic'] }, "&&Reopen") : localize({ key: 'newWindow', comment: ['&& denotes a mnemonic'] }, "&&New Window"),
-								localize({ key: 'close', comment: ['&& denotes a mnemonic'] }, "&&Close")
-							],
-							message,
-							detail: this._config?.workspace ?
-								localize('appGoneDetailWorkspace', "We are sorry for the inconvenience. You can reopen the window to continue where you left off.") :
-								localize('appGoneDetailEmptyWindow', "We are sorry for the inconvenience. You can open a new empty window to start again."),
-							checkboxLabel: this._config?.workspace ? localize('doNotRestoreEditors', "Don't restore editors") : undefined
-						}, this._win);
-
-						// Handle choice
-						const reopen = response === 0;
-						await this.destroyWindow(reopen, checkboxChecked);
-					}
+					// Handle choice
+					const reopen = response === 0;
+					await this.destroyWindow(reopen, checkboxChecked);
 				}
 				break;
 		}
 	}
 
-	private async handleWindowsAdminCrash(details: { reason?: string; exitCode?: number; sandboxed: boolean }) {
-
-		// Prepare telemetry event (TODO@bpasero remove me eventually)
-		const appenders: ITelemetryAppender[] = [];
-		const isInternal = isInternalTelemetry(this.productService, this.configurationService);
-		if (supportsTelemetry(this.productService, this.environmentMainService)) {
-			if (this.productService.aiConfig && this.productService.aiConfig.ariaKey) {
-				appenders.push(new OneDataSystemAppender(isInternal, 'monacoworkbench', null, this.productService.aiConfig.ariaKey));
-			}
-
-			const machineId = await resolveMachineId(this.stateService, this.logService);
-
-			const config: ITelemetryServiceConfig = {
-				appenders,
-				sendErrorTelemetry: false,
-				commonProperties: resolveCommonProperties(release(), hostname(), process.arch, this.productService.commit, this.productService.version, machineId, isInternal),
-				piiPaths: getPiiPathsFromEnvironment(this.environmentMainService)
-			};
-
-			const telemetryService = new TelemetryService(config, this.configurationService, this.productService);
-
-			type WindowAdminErrorClassification = {
-				reason: { classification: 'SystemMetaData'; purpose: 'PerformanceAndHealth'; comment: 'The reason of the window error to understand the nature of the error better.' };
-				code: { classification: 'SystemMetaData'; purpose: 'PerformanceAndHealth'; isMeasurement: true; comment: 'The exit code of the window process to understand the nature of the error better' };
-				owner: 'bpasero';
-				comment: 'Provides insight into reasons the vscode window had an error when running as admin.';
-			};
-			type WindowAdminErrorEvent = {
-				reason: string | undefined;
-				code: number | undefined;
-			};
-			await telemetryService.publicLog2<WindowAdminErrorEvent, WindowAdminErrorClassification>('windowadminerror', { reason: details.reason, code: details.exitCode });
-		}
-
-		// Inform user
-		const { response } = await this.dialogMainService.showMessageBox({
-			type: 'error',
-			buttons: [
-				localize({ key: 'learnMore', comment: ['&& denotes a mnemonic'] }, "&&Learn More"),
-				localize({ key: 'close', comment: ['&& denotes a mnemonic'] }, "&&Close")
-			],
-			message: localize('appGoneAdminMessage', "Running as administrator is not supported in your environment"),
-			detail: localize('appGoneAdminDetail', "We are sorry for the inconvenience. Please try again without administrator privileges.", this.productService.nameLong)
-		}, this._win);
-
-		if (response === 0) {
-			await this.nativeHostMainService.openExternal(undefined, 'https://go.microsoft.com/fwlink/?linkid=2220179');
-		}
-
-		// Ensure to await flush telemetry
-		await Promise.all(appenders.map(appender => appender.flush()));
-
-		// Exit
-		await this.destroyWindow(false, false);
-	}
-
 	private async destroyWindow(reopen: boolean, skipRestoreEditors: boolean): Promise<void> {
 		const workspace = this._config?.workspace;
 
