@@ -17,18 +17,18 @@ export const defaultQuickAccessContextKeyValue = 'inFilesPicker';
 export const defaultQuickAccessContext = ContextKeyExpr.and(inQuickPickContext, ContextKeyExpr.has(defaultQuickAccessContextKeyValue));
 
 export interface IWorkbenchQuickAccessConfiguration {
-	workbench: {
-		commandPalette: {
-			history: number;
-			preserveInput: boolean;
-			experimental: {
-				suggestCommands: boolean;
-				useSemanticSimilarity: boolean;
+	readonly workbench: {
+		readonly commandPalette: {
+			readonly history: number;
+			readonly preserveInput: boolean;
+			readonly experimental: {
+				readonly suggestCommands: boolean;
+				readonly useSemanticSimilarity: boolean;
 			};
 		};
-		quickOpen: {
-			enableExperimentalNewVersion: boolean;
-			preserveInput: boolean;
+		readonly quickOpen: {
+			readonly enableExperimentalNewVersion: boolean;
+			readonly preserveInput: boolean;
 		};
 	};
 }
