@@ -23,11 +23,11 @@ import { EditorOption } from 'vs/editor/common/config/editorOptions';
 import { ServiceCollection } from 'vs/platform/instantiation/common/serviceCollection';
 import { CommentThreadWidget } from 'vs/workbench/contrib/comments/browser/commentThreadWidget';
 import { ICellRange } from 'vs/workbench/contrib/notebook/common/notebookRange';
-import { commentThreadStateBackgroundColorVar, commentThreadStateColorVar, getCommentThreadStateColor } from 'vs/workbench/contrib/comments/browser/commentColors';
+import { commentThreadStateBackgroundColorVar, commentThreadStateColorVar, getCommentThreadStateBorderColor } from 'vs/workbench/contrib/comments/browser/commentColors';
 import { peekViewBorder } from 'vs/editor/contrib/peekView/browser/peekView';
 
 function getCommentThreadWidgetStateColor(thread: languages.CommentThreadState | undefined, theme: IColorTheme): Color | undefined {
-	return getCommentThreadStateColor(thread, theme) ?? theme.getColor(peekViewBorder);
+	return getCommentThreadStateBorderColor(thread, theme) ?? theme.getColor(peekViewBorder);
 }
 
 export function parseMouseDownInfoFromEvent(e: IEditorMouseEvent) {
@@ -416,8 +416,12 @@ export class ReviewZoneWidget extends ZoneWidget implements ICommentThreadWidget
 		this._commentThreadWidget.submitComment();
 	}
 
-	_refresh(dimensions?: dom.Dimension) {
-		if (this._isExpanded && dimensions) {
+	_refresh(dimensions: dom.Dimension) {
+		if (dimensions.height === 0 && dimensions.width === 0) {
+			this.commentThread.collapsibleState = languages.CommentThreadCollapsibleState.Collapsed;
+			return;
+		}
+		if (this._isExpanded) {
 			this._commentThreadWidget.layout();
 
 			const headHeight = Math.ceil(this.editor.getOption(EditorOption.lineHeight) * 1.2);
