@@ -133,11 +133,13 @@ export class ConfigurationManager implements IConfigurationManager {
 			return config;
 		};
 
+		let resolvedType = config.type ?? type;
 		let result: IConfig | null | undefined = config;
-		for (let seen = new Set(); result && !seen.has(result.type);) {
-			seen.add(result?.type);
-			result = await resolveDebugConfigurationForType(result.type, result);
+		for (let seen = new Set(); result && !seen.has(resolvedType);) {
+			seen.add(resolvedType);
+			result = await resolveDebugConfigurationForType(resolvedType, result);
 			result = await resolveDebugConfigurationForType('*', result);
+			resolvedType = result?.type ?? type!;
 		}
 
 		return result;
@@ -207,24 +209,6 @@ export class ConfigurationManager implements IConfigurationManager {
 				pick: async () => {
 					// Do a late 'onDebugDynamicConfigurationsName' activation so extensions are not activated too early #108578
 					await this.adapterManager.activateDebuggers(onDebugDynamicConfigurationsName, type);
-					const disposables = new DisposableStore();
-					const input = disposables.add(this.quickInputService.createQuickPick<IDynamicPickItem>());
-					input.busy = true;
-					input.placeholder = nls.localize('selectConfiguration', "Select Launch Configuration");
-					input.show();
-
-					const chosenPromise = new Promise<IDynamicPickItem | undefined>(resolve => {
-						disposables.add(input.onDidAccept(() => resolve(input.activeItems[0])));
-						disposables.add(input.onDidTriggerItemButton(async (context) => {
-							resolve(undefined);
-							const { launch, config } = context.item;
-							await launch.openConfigFile({ preserveFocus: false, type: config.type, suppressInitialConfigs: true });
-							// Only Launch have a pin trigger button
-							await (launch as Launch).writeConfiguration(config);
-							await this.selectConfiguration(launch, config.name);
-							this.removeRecentDynamicConfigurations(config.name, config.type);
-						}));
-					});
 
 					const token = new CancellationTokenSource();
 					const picks: Promise<IDynamicPickItem[]>[] = [];
@@ -244,11 +228,30 @@ export class ConfigurationManager implements IConfigurationManager {
 						}
 					});
 
+					const disposables = new DisposableStore();
+					const input = disposables.add(this.quickInputService.createQuickPick<IDynamicPickItem>());
+					input.busy = true;
+					input.placeholder = nls.localize('selectConfiguration', "Select Launch Configuration");
+
+					const chosenPromise = new Promise<IDynamicPickItem | undefined>(resolve => {
+						disposables.add(input.onDidAccept(() => resolve(input.activeItems[0])));
+						disposables.add(input.onDidTriggerItemButton(async (context) => {
+							resolve(undefined);
+							const { launch, config } = context.item;
+							await launch.openConfigFile({ preserveFocus: false, type: config.type, suppressInitialConfigs: true });
+							// Only Launch have a pin trigger button
+							await (launch as Launch).writeConfiguration(config);
+							await this.selectConfiguration(launch, config.name);
+							this.removeRecentDynamicConfigurations(config.name, config.type);
+						}));
+					});
+
 					const nestedPicks = await Promise.all(picks);
 					const items = flatten(nestedPicks);
 
 					input.items = items;
 					input.busy = false;
+					input.show();
 					const chosen = await chosenPromise;
 
 					disposables.dispose();
@@ -281,7 +284,7 @@ export class ConfigurationManager implements IConfigurationManager {
 
 	removeRecentDynamicConfigurations(name: string, type: string) {
 		const remaining = this.getRecentDynamicConfigurations().filter(c => c.name !== name || c.type !== type);
-		this.storageService.store(DEBUG_RECENT_DYNAMIC_CONFIGURATIONS, JSON.stringify(remaining), StorageScope.WORKSPACE, StorageTarget.USER);
+		this.storageService.store(DEBUG_RECENT_DYNAMIC_CONFIGURATIONS, JSON.stringify(remaining), StorageScope.WORKSPACE, StorageTarget.MACHINE);
 		if (this.selectedConfiguration.name === name && this.selectedType === type && this.selectedDynamic) {
 			this.selectConfiguration(undefined, undefined);
 		} else {
@@ -426,7 +429,7 @@ export class ConfigurationManager implements IConfigurationManager {
 				// We need to store the recently used dynamic configurations to be able to show them in UI #110009
 				recentDynamicProviders.unshift({ name, type: dynamicConfig.type });
 				recentDynamicProviders = distinct(recentDynamicProviders, t => `${t.name} : ${t.type}`);
-				this.storageService.store(DEBUG_RECENT_DYNAMIC_CONFIGURATIONS, JSON.stringify(recentDynamicProviders), StorageScope.WORKSPACE, StorageTarget.USER);
+				this.storageService.store(DEBUG_RECENT_DYNAMIC_CONFIGURATIONS, JSON.stringify(recentDynamicProviders), StorageScope.WORKSPACE, StorageTarget.MACHINE);
 			}
 		} else if (!this.selectedName || names.indexOf(this.selectedName) === -1) {
 			// We could not find the configuration to select, pick the first one, or reset the selection if there is no launch configuration
