@@ -14,12 +14,15 @@ import { registerIcon } from 'vs/platform/theme/common/iconRegistry';
 import { IResourceRefHandle } from 'vs/platform/userDataSync/common/userDataSync';
 import { Event } from 'vs/base/common/event';
 import { StringSHA1 } from 'vs/base/common/hash';
+import { EditSessionsStoreClient } from 'vs/workbench/contrib/editSessions/common/editSessionsStorageClient';
 
 export const EDIT_SESSION_SYNC_CATEGORY: ILocalizedString = {
 	original: 'Cloud Changes',
 	value: localize('cloud changes', 'Cloud Changes')
 };
 
+export type SyncResource = 'editSessions' | 'workspaceState';
+
 export const IEditSessionsStorageService = createDecorator<IEditSessionsStorageService>('IEditSessionsStorageService');
 export interface IEditSessionsStorageService {
 	_serviceBrand: undefined;
@@ -30,11 +33,13 @@ export interface IEditSessionsStorageService {
 	readonly onDidSignIn: Event<void>;
 	readonly onDidSignOut: Event<void>;
 
+	storeClient: EditSessionsStoreClient | undefined;
+
 	initialize(silent?: boolean): Promise<boolean>;
-	read(ref: string | undefined): Promise<{ ref: string; editSession: EditSession } | undefined>;
-	write(editSession: EditSession): Promise<string>;
-	delete(ref: string | null): Promise<void>;
-	list(): Promise<IResourceRefHandle[]>;
+	read(resource: SyncResource, ref: string | undefined): Promise<{ ref: string; content: string } | undefined>;
+	write(resource: SyncResource, content: string | EditSession): Promise<string>;
+	delete(resource: SyncResource, ref: string | null): Promise<void>;
+	list(resource: SyncResource): Promise<IResourceRefHandle[]>;
 	getMachineById(machineId: string): Promise<string | undefined>;
 }
 
@@ -79,7 +84,6 @@ export interface EditSession {
 	version: number;
 	machine?: string;
 	folders: Folder[];
-	state: { [key: string]: unknown };
 }
 
 export const EDIT_SESSIONS_SIGNED_IN_KEY = 'editSessionsSignedIn';
