@@ -6,10 +6,12 @@
 import { CompareResult, equals } from 'vs/base/common/arrays';
 import { BugIndicatingError } from 'vs/base/common/errors';
 import { autorunHandleChanges, derived, IObservable, IReader, ISettableObservable, ITransaction, keepAlive, observableValue, transaction, waitForState } from 'vs/base/common/observable';
+import { URI } from 'vs/base/common/uri';
 import { Range } from 'vs/editor/common/core/range';
 import { ILanguageService } from 'vs/editor/common/languages/language';
 import { ITextModel } from 'vs/editor/common/model';
-import { IModelService } from 'vs/editor/common/services/model';
+import { localize } from 'vs/nls';
+import { IResourceUndoRedoElement, IUndoRedoService, UndoRedoElementType, UndoRedoGroup } from 'vs/platform/undoRedo/common/undoRedo';
 import { EditorModel } from 'vs/workbench/common/editor/editorModel';
 import { IMergeDiffComputer } from 'vs/workbench/contrib/mergeEditor/browser/model/diffComputer';
 import { LineRange } from 'vs/workbench/contrib/mergeEditor/browser/model/lineRange';
@@ -17,7 +19,7 @@ import { DetailedLineRangeMapping, DocumentLineRangeMap, DocumentRangeMap, LineR
 import { TextModelDiffChangeReason, TextModelDiffs, TextModelDiffState } from 'vs/workbench/contrib/mergeEditor/browser/model/textModelDiffs';
 import { MergeEditorTelemetry } from 'vs/workbench/contrib/mergeEditor/browser/telemetry';
 import { leftJoin } from 'vs/workbench/contrib/mergeEditor/browser/utils';
-import { ModifiedBaseRange, ModifiedBaseRangeState, ModifiedBaseRangeStateKind } from './modifiedBaseRange';
+import { InputNumber, ModifiedBaseRange, ModifiedBaseRangeState, ModifiedBaseRangeStateKind } from './modifiedBaseRange';
 
 export interface InputData {
 	readonly textModel: ITextModel;
@@ -29,7 +31,7 @@ export interface InputData {
 export class MergeEditorModel extends EditorModel {
 	private readonly input1TextModelDiffs = this._register(new TextModelDiffs(this.base, this.input1.textModel, this.diffComputer));
 	private readonly input2TextModelDiffs = this._register(new TextModelDiffs(this.base, this.input2.textModel, this.diffComputer));
-	private readonly resultTextModelDiffs = this._register(new TextModelDiffs(this.base, this.resultTextModel, this.diffComputerConflictProjection));
+	private readonly resultTextModelDiffs = this._register(new TextModelDiffs(this.base, this.resultTextModel, this.diffComputer));
 	public readonly modifiedBaseRanges = derived<ModifiedBaseRange[]>('modifiedBaseRanges', (reader) => {
 		const input1Diffs = this.input1TextModelDiffs.diffs.read(reader);
 		const input2Diffs = this.input2TextModelDiffs.diffs.read(reader);
@@ -40,13 +42,7 @@ export class MergeEditorModel extends EditorModel {
 		derived('modifiedBaseRangeResultStates', reader => {
 			const map = new Map<ModifiedBaseRange, ModifiedBaseRangeData>(
 				this.modifiedBaseRanges.read(reader).map<[ModifiedBaseRange, ModifiedBaseRangeData]>((s) => [
-					s,
-					{
-						accepted: observableValue(`BaseRangeState${s.baseRange}`, ModifiedBaseRangeState.base),
-						handled: observableValue(`BaseRangeHandledState${s.baseRange}`, false),
-						computedFromDiffing: false,
-						previousNonDiffingState: undefined,
-					}
+					s, new ModifiedBaseRangeData(s)
 				])
 			);
 			return map;
@@ -60,11 +56,10 @@ export class MergeEditorModel extends EditorModel {
 		readonly input2: InputData,
 		readonly resultTextModel: ITextModel,
 		private readonly diffComputer: IMergeDiffComputer,
-		private readonly diffComputerConflictProjection: IMergeDiffComputer,
 		private readonly options: { resetResult: boolean },
 		public readonly telemetry: MergeEditorTelemetry,
-		@IModelService private readonly modelService: IModelService,
 		@ILanguageService private readonly languageService: ILanguageService,
+		@IUndoRedoService private readonly undoRedoService: IUndoRedoService,
 	) {
 		super();
 
@@ -109,7 +104,9 @@ export class MergeEditorModel extends EditorModel {
 								shouldRecomputeHandledFromAccepted = false;
 								for (const [_range, observableState] of states) {
 									const state = observableState.accepted.get();
-									observableState.handled.set(!(state.kind === ModifiedBaseRangeStateKind.base || state.kind === ModifiedBaseRangeStateKind.unrecognized), tx);
+									const handled = !(state.kind === ModifiedBaseRangeStateKind.base || state.kind === ModifiedBaseRangeStateKind.unrecognized);
+									observableState.handledInput1.set(handled, tx);
+									observableState.handledInput2.set(handled, tx);
 								}
 							}
 						});
@@ -141,6 +138,9 @@ export class MergeEditorModel extends EditorModel {
 				} else if (range.input2Diffs.length === 0) {
 					newState = ModifiedBaseRangeState.base.withInputValue(1, true);
 					handled = true;
+				} else if (range.isEqualChange) {
+					newState = ModifiedBaseRangeState.base.withInputValue(1, true);
+					handled = true;
 				} else {
 					newState = ModifiedBaseRangeState.base;
 					handled = false;
@@ -149,10 +149,14 @@ export class MergeEditorModel extends EditorModel {
 				state.accepted.set(newState, tx);
 				state.computedFromDiffing = false;
 				state.previousNonDiffingState = undefined;
-				state.handled.set(handled, tx);
+				state.handledInput1.set(handled, tx);
+				state.handledInput2.set(handled, tx);
 			}
 
-			this.resultTextModel.setValue(this.computeAutoMergedResult());
+			this.resultTextModel.pushEditOperations(null, [{
+				range: new Range(1, 1, Number.MAX_SAFE_INTEGER, 1),
+				text: this.computeAutoMergedResult()
+			}], () => null);
 		});
 	}
 
@@ -180,6 +184,8 @@ export class MergeEditorModel extends EditorModel {
 				appendLinesToResult(input2Lines, baseRange.input2Range);
 			} else if (baseRange.input2Diffs.length === 0) {
 				appendLinesToResult(input1Lines, baseRange.input1Range);
+			} else if (baseRange.isEqualChange) {
+				appendLinesToResult(input1Lines, baseRange.input1Range);
 			} else {
 				appendLinesToResult(baseLines, baseRange.baseRange);
 			}
@@ -198,6 +204,7 @@ export class MergeEditorModel extends EditorModel {
 
 	public readonly baseInput2Diffs = this.input2TextModelDiffs.diffs;
 	public readonly baseResultDiffs = this.resultTextModelDiffs.diffs;
+	public get isApplyingEditInResult(): boolean { return this.resultTextModelDiffs.isApplyingChange; }
 	public readonly input1ResultMapping = derived('input1ResultMapping', reader => {
 		return this.getInputResultMapping(
 			this.baseInput1Diffs.read(reader),
@@ -405,9 +412,9 @@ export class MergeEditorModel extends EditorModel {
 	public setState(
 		baseRange: ModifiedBaseRange,
 		state: ModifiedBaseRangeState,
-		markHandled: boolean,
-		transaction: ITransaction,
-		pushStackElement: boolean = false
+		_markInputAsHandled: boolean | InputNumber,
+		tx: ITransaction,
+		_pushStackElement: boolean = false
 	): void {
 		if (!this.isUpToDate.get()) {
 			throw new BugIndicatingError('Cannot set state while updating');
@@ -421,29 +428,36 @@ export class MergeEditorModel extends EditorModel {
 		const conflictingDiffs = this.resultTextModelDiffs.findTouchingDiffs(
 			baseRange.baseRange
 		);
+		const group = new UndoRedoGroup();
 		if (conflictingDiffs) {
-			this.resultTextModelDiffs.removeDiffs(conflictingDiffs, transaction);
+			this.resultTextModelDiffs.removeDiffs(conflictingDiffs, tx, group);
 		}
 
 		const { edit, effectiveState } = baseRange.getEditForBase(state);
 
-		existingState.accepted.set(effectiveState, transaction);
+		existingState.accepted.set(effectiveState, tx);
 		existingState.previousNonDiffingState = undefined;
 		existingState.computedFromDiffing = false;
 
-		if (edit) {
-			if (pushStackElement) {
-				this.resultTextModel.pushStackElement();
-			}
-			this.resultTextModelDiffs.applyEditRelativeToOriginal(edit, transaction);
-			if (pushStackElement) {
-				this.resultTextModel.pushStackElement();
-			}
+		const input1Handled = existingState.handledInput1.get();
+		const input2Handled = existingState.handledInput2.get();
+
+		if (!input1Handled || !input2Handled) {
+			this.undoRedoService.pushElement(
+				new MarkAsHandledUndoRedoElement(this.resultTextModel.uri, new WeakRef(this), new WeakRef(existingState), input1Handled, input2Handled),
+				group
+			);
 		}
 
-		if (markHandled) {
-			existingState.handled.set(true, transaction);
+		if (edit) {
+			this.resultTextModel.pushStackElement();
+			this.resultTextModelDiffs.applyEditRelativeToOriginal(edit, tx, group);
+			this.resultTextModel.pushStackElement();
 		}
+
+		// always set conflict as handled
+		existingState.handledInput1.set(true, tx);
+		existingState.handledInput2.set(true, tx);
 	}
 
 	public resetDirtyConflictsToBase(): void {
@@ -463,13 +477,68 @@ export class MergeEditorModel extends EditorModel {
 		return this.modifiedBaseRangeResultStates.get().get(baseRange)!.handled;
 	}
 
+	public isInputHandled(baseRange: ModifiedBaseRange, inputNumber: InputNumber): IObservable<boolean> {
+		const state = this.modifiedBaseRangeResultStates.get().get(baseRange)!;
+		return inputNumber === 1 ? state.handledInput1 : state.handledInput2;
+	}
+
+	public setInputHandled(baseRange: ModifiedBaseRange, inputNumber: InputNumber, handled: boolean, tx: ITransaction): void {
+		const state = this.modifiedBaseRangeResultStates.get().get(baseRange)!;
+		if (state.handled.get() === handled) {
+			return;
+		}
+
+		const dataRef = new WeakRef(ModifiedBaseRangeData);
+		const modelRef = new WeakRef(this);
+
+		this.undoRedoService.pushElement({
+			type: UndoRedoElementType.Resource,
+			resource: this.resultTextModel.uri,
+			code: 'setInputHandled',
+			label: localize('setInputHandled', "Set Input Handled"),
+			redo() {
+				const model = modelRef.deref();
+				const data = dataRef.deref();
+				if (model && !model.isDisposed() && data) {
+					transaction(tx => {
+						if (inputNumber === 1) {
+							state.handledInput1.set(handled, tx);
+						} else {
+							state.handledInput2.set(handled, tx);
+						}
+					});
+				}
+			},
+			undo() {
+				const model = modelRef.deref();
+				const data = dataRef.deref();
+				if (model && !model.isDisposed() && data) {
+					transaction(tx => {
+						if (inputNumber === 1) {
+							state.handledInput1.set(!handled, tx);
+						} else {
+							state.handledInput2.set(!handled, tx);
+						}
+					});
+				}
+			},
+		});
+
+		if (inputNumber === 1) {
+			state.handledInput1.set(handled, tx);
+		} else {
+			state.handledInput2.set(handled, tx);
+		}
+	}
+
 	public setHandled(baseRange: ModifiedBaseRange, handled: boolean, tx: ITransaction): void {
 		const state = this.modifiedBaseRangeResultStates.get().get(baseRange)!;
 		if (state.handled.get() === handled) {
 			return;
 		}
 
-		state.handled.set(handled, tx);
+		state.handledInput1.set(handled, tx);
+		state.handledInput2.set(handled, tx);
 	}
 
 	public readonly unhandledConflictsCount = derived('unhandledConflictsCount', reader => {
@@ -487,10 +556,10 @@ export class MergeEditorModel extends EditorModel {
 
 	public setLanguageId(languageId: string, source?: string): void {
 		const language = this.languageService.createById(languageId);
-		this.modelService.setMode(this.base, language, source);
-		this.modelService.setMode(this.input1.textModel, language, source);
-		this.modelService.setMode(this.input2.textModel, language, source);
-		this.modelService.setMode(this.resultTextModel, language, source);
+		this.base.setLanguage(language, source);
+		this.input1.textModel.setLanguage(language, source);
+		this.input2.textModel.setLanguage(language, source);
+		this.resultTextModel.setLanguage(language, source);
 	}
 
 	public getInitialResultValue(): string {
@@ -686,13 +755,17 @@ function arrayCount<T>(array: Iterable<T>, predicate: (value: T) => boolean): nu
 	return count;
 }
 
-interface ModifiedBaseRangeData {
-	accepted: ISettableObservable<ModifiedBaseRangeState>;
+class ModifiedBaseRangeData {
+	constructor(private readonly baseRange: ModifiedBaseRange) { }
 
-	handled: ISettableObservable<boolean>;
+	public accepted: ISettableObservable<ModifiedBaseRangeState> = observableValue(`BaseRangeState${this.baseRange.baseRange}`, ModifiedBaseRangeState.base);
+	public handledInput1: ISettableObservable<boolean> = observableValue(`BaseRangeHandledState${this.baseRange.baseRange}.Input1`, false);
+	public handledInput2: ISettableObservable<boolean> = observableValue(`BaseRangeHandledState${this.baseRange.baseRange}.Input2`, false);
 
-	computedFromDiffing: boolean;
-	previousNonDiffingState: ModifiedBaseRangeState | undefined;
+	public computedFromDiffing = false;
+	public previousNonDiffingState: ModifiedBaseRangeState | undefined = undefined;
+
+	public readonly handled = derived('handled', reader => this.handledInput1.read(reader) && this.handledInput2.read(reader));
 }
 
 export const enum MergeEditorModelState {
@@ -700,3 +773,43 @@ export const enum MergeEditorModelState {
 	upToDate = 2,
 	updating = 3,
 }
+
+class MarkAsHandledUndoRedoElement implements IResourceUndoRedoElement {
+	public readonly code = 'undoMarkAsHandled';
+	public readonly label = localize('undoMarkAsHandled', 'Undo Mark As Handled');
+
+	public readonly type = UndoRedoElementType.Resource;
+
+	constructor(
+		public readonly resource: URI,
+		private readonly mergeEditorModelRef: WeakRef<MergeEditorModel>,
+		private readonly stateRef: WeakRef<ModifiedBaseRangeData>,
+		private readonly input1Handled: boolean,
+		private readonly input2Handled: boolean,
+	) { }
+
+	public redo() {
+		const mergeEditorModel = this.mergeEditorModelRef.deref();
+		if (!mergeEditorModel || mergeEditorModel.isDisposed()) {
+			return;
+		}
+		const state = this.stateRef.deref();
+		if (!state) { return; }
+		transaction(tx => {
+			state.handledInput1.set(true, tx);
+			state.handledInput2.set(true, tx);
+		});
+	}
+	public undo() {
+		const mergeEditorModel = this.mergeEditorModelRef.deref();
+		if (!mergeEditorModel || mergeEditorModel.isDisposed()) {
+			return;
+		}
+		const state = this.stateRef.deref();
+		if (!state) { return; }
+		transaction(tx => {
+			state.handledInput1.set(this.input1Handled, tx);
+			state.handledInput2.set(this.input2Handled, tx);
+		});
+	}
+}
