@@ -5,6 +5,8 @@
 
 import { timeout } from 'vs/base/common/async';
 import { KeyCode, KeyMod } from 'vs/base/common/keyCodes';
+import { isEqual } from 'vs/base/common/resources';
+import { ICodeEditor } from 'vs/editor/browser/editorBrowser';
 import { EditorExtensionsRegistry } from 'vs/editor/browser/editorExtensions';
 import { EditorContextKeys } from 'vs/editor/common/editorContextKeys';
 import { localize } from 'vs/nls';
@@ -16,8 +18,10 @@ import { InputFocusedContextKey } from 'vs/platform/contextkey/common/contextkey
 import { ServicesAccessor } from 'vs/platform/instantiation/common/instantiation';
 import { KeybindingWeight } from 'vs/platform/keybinding/common/keybindingsRegistry';
 import { Registry } from 'vs/platform/registry/common/platform';
+import { InteractiveEditorController } from 'vs/workbench/contrib/interactiveEditor/browser/interactiveEditorController';
+import { CTX_INTERACTIVE_EDITOR_FOCUSED, CTX_INTERACTIVE_EDITOR_INNER_CURSOR_LAST } from 'vs/workbench/contrib/interactiveEditor/common/interactiveEditor';
 import { INotebookActionContext, INotebookCellActionContext, NotebookAction, NotebookCellAction, NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT } from 'vs/workbench/contrib/notebook/browser/controller/coreActions';
-import { CellEditState } from 'vs/workbench/contrib/notebook/browser/notebookBrowser';
+import { CellEditState, ICellViewModel } from 'vs/workbench/contrib/notebook/browser/notebookBrowser';
 import { CellKind, NOTEBOOK_EDITOR_CURSOR_BOUNDARY } from 'vs/workbench/contrib/notebook/common/notebookCommon';
 import { NOTEBOOK_CELL_HAS_OUTPUTS, NOTEBOOK_CELL_MARKDOWN_EDIT_MODE, NOTEBOOK_CELL_TYPE, NOTEBOOK_CURSOR_NAVIGATION_MODE, NOTEBOOK_EDITOR_FOCUSED, NOTEBOOK_OUTPUT_FOCUSED } from 'vs/workbench/contrib/notebook/common/notebookContextKeys';
 
@@ -33,6 +37,17 @@ const NOTEBOOK_CURSOR_PAGEUP_SELECT_COMMAND_ID = 'notebook.cell.cursorPageUpSele
 const NOTEBOOK_CURSOR_PAGEDOWN_COMMAND_ID = 'notebook.cell.cursorPageDown';
 const NOTEBOOK_CURSOR_PAGEDOWN_SELECT_COMMAND_ID = 'notebook.cell.cursorPageDownSelect';
 
+function findTargetCellEditor(context: INotebookCellActionContext, targetCell: ICellViewModel) {
+	let foundEditor: ICodeEditor | undefined = undefined;
+	for (const [, codeEditor] of context.notebookEditor.codeEditors) {
+		if (isEqual(codeEditor.getModel()?.uri, targetCell.uri)) {
+			foundEditor = codeEditor;
+			break;
+		}
+	}
+
+	return foundEditor;
+}
 
 registerAction2(class FocusNextCellAction extends NotebookCellAction {
 	constructor() {
@@ -51,6 +66,7 @@ registerAction2(class FocusNextCellAction extends NotebookCellAction {
 							NOTEBOOK_EDITOR_CURSOR_BOUNDARY.notEqualsTo('top'),
 							NOTEBOOK_EDITOR_CURSOR_BOUNDARY.notEqualsTo('none'),
 						),
+						EditorContextKeys.isEmbeddedDiffEditor.negate()
 					),
 					primary: KeyCode.DownArrow,
 					weight: NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT, // code cell keybinding, focus inside editor: lower weight to not override suggest widget
@@ -63,7 +79,8 @@ registerAction2(class FocusNextCellAction extends NotebookCellAction {
 						ContextKeyExpr.and(
 							NOTEBOOK_CELL_TYPE.isEqualTo('markup'),
 							NOTEBOOK_CELL_MARKDOWN_EDIT_MODE.isEqualTo(false),
-							NOTEBOOK_CURSOR_NAVIGATION_MODE)
+							NOTEBOOK_CURSOR_NAVIGATION_MODE),
+						EditorContextKeys.isEmbeddedDiffEditor.negate()
 					),
 					primary: KeyCode.DownArrow,
 					weight: KeybindingWeight.WorkbenchContrib, // markdown keybinding, focus on list: higher weight to override list.focusDown
@@ -73,6 +90,39 @@ registerAction2(class FocusNextCellAction extends NotebookCellAction {
 					primary: KeyMod.CtrlCmd | KeyCode.DownArrow,
 					mac: { primary: KeyMod.WinCtrl | KeyMod.CtrlCmd | KeyCode.DownArrow, },
 					weight: KeybindingWeight.WorkbenchContrib
+				},
+				{
+					when: ContextKeyExpr.and(
+						NOTEBOOK_EDITOR_FOCUSED,
+						CONTEXT_ACCESSIBILITY_MODE_ENABLED.negate(),
+						ContextKeyExpr.equals('config.notebook.navigation.allowNavigateToSurroundingCells', true),
+						ContextKeyExpr.and(
+							ContextKeyExpr.has(InputFocusedContextKey),
+							NOTEBOOK_EDITOR_CURSOR_BOUNDARY.notEqualsTo('top'),
+							NOTEBOOK_EDITOR_CURSOR_BOUNDARY.notEqualsTo('none'),
+						),
+						CTX_INTERACTIVE_EDITOR_FOCUSED,
+						CTX_INTERACTIVE_EDITOR_INNER_CURSOR_LAST,
+						EditorContextKeys.isEmbeddedDiffEditor.negate()
+					),
+					primary: KeyCode.DownArrow,
+					weight: KeybindingWeight.EditorCore
+				},
+				{
+					when: ContextKeyExpr.and(
+						NOTEBOOK_EDITOR_FOCUSED,
+						CONTEXT_ACCESSIBILITY_MODE_ENABLED.negate(),
+						ContextKeyExpr.equals('config.notebook.navigation.allowNavigateToSurroundingCells', true),
+						ContextKeyExpr.and(
+							NOTEBOOK_CELL_TYPE.isEqualTo('markup'),
+							NOTEBOOK_CELL_MARKDOWN_EDIT_MODE.isEqualTo(false),
+							NOTEBOOK_CURSOR_NAVIGATION_MODE),
+						CTX_INTERACTIVE_EDITOR_FOCUSED,
+						CTX_INTERACTIVE_EDITOR_INNER_CURSOR_LAST,
+						EditorContextKeys.isEmbeddedDiffEditor.negate()
+					),
+					primary: KeyCode.DownArrow,
+					weight: KeybindingWeight.EditorCore
 				}
 			]
 		});
@@ -92,9 +142,17 @@ registerAction2(class FocusNextCellAction extends NotebookCellAction {
 			return;
 		}
 
-		const newCell = editor.cellAt(idx + 1);
-		const newFocusMode = newCell.cellKind === CellKind.Markup && newCell.getEditState() === CellEditState.Preview ? 'container' : 'editor';
-		await editor.focusNotebookCell(newCell, newFocusMode, { focusEditorLine: 1 });
+		const focusEditorLine = activeCell.textBuffer.getLineCount();
+		const targetCell = (context.cell ?? context.selectedCells?.[0]);
+		const foundEditor: ICodeEditor | undefined = targetCell ? findTargetCellEditor(context, targetCell) : undefined;
+
+		if (foundEditor && foundEditor.hasTextFocus() && InteractiveEditorController.get(foundEditor)?.getWidgetPosition()?.lineNumber === focusEditorLine) {
+			InteractiveEditorController.get(foundEditor)?.focus();
+		} else {
+			const newCell = editor.cellAt(idx + 1);
+			const newFocusMode = newCell.cellKind === CellKind.Markup && newCell.getEditState() === CellEditState.Preview ? 'container' : 'editor';
+			await editor.focusNotebookCell(newCell, newFocusMode, { focusEditorLine: 1 });
+		}
 	}
 });
 
@@ -117,6 +175,7 @@ registerAction2(class FocusPreviousCellAction extends NotebookCellAction {
 							NOTEBOOK_EDITOR_CURSOR_BOUNDARY.notEqualsTo('bottom'),
 							NOTEBOOK_EDITOR_CURSOR_BOUNDARY.notEqualsTo('none'),
 						),
+						EditorContextKeys.isEmbeddedDiffEditor.negate()
 					),
 					primary: KeyCode.UpArrow,
 					weight: NOTEBOOK_EDITOR_WIDGET_ACTION_WEIGHT, // code cell keybinding, focus inside editor: lower weight to not override suggest widget
@@ -130,7 +189,8 @@ registerAction2(class FocusPreviousCellAction extends NotebookCellAction {
 							NOTEBOOK_CELL_TYPE.isEqualTo('markup'),
 							NOTEBOOK_CELL_MARKDOWN_EDIT_MODE.isEqualTo(false),
 							NOTEBOOK_CURSOR_NAVIGATION_MODE
-						)
+						),
+						EditorContextKeys.isEmbeddedDiffEditor.negate()
 					),
 					primary: KeyCode.UpArrow,
 					weight: KeybindingWeight.WorkbenchContrib, // markdown keybinding, focus on list: higher weight to override list.focusDown
@@ -155,7 +215,14 @@ registerAction2(class FocusPreviousCellAction extends NotebookCellAction {
 
 		const newCell = editor.cellAt(idx - 1);
 		const newFocusMode = newCell.cellKind === CellKind.Markup && newCell.getEditState() === CellEditState.Preview ? 'container' : 'editor';
-		await editor.focusNotebookCell(newCell, newFocusMode, { focusEditorLine: newCell.textBuffer.getLineCount() });
+		const focusEditorLine = newCell.textBuffer.getLineCount();
+		await editor.focusNotebookCell(newCell, newFocusMode, { focusEditorLine: focusEditorLine });
+
+		const foundEditor: ICodeEditor | undefined = findTargetCellEditor(context, newCell);
+
+		if (foundEditor && InteractiveEditorController.get(foundEditor)?.getWidgetPosition()?.lineNumber === focusEditorLine) {
+			InteractiveEditorController.get(foundEditor)?.focus();
+		}
 	}
 });
 
