@@ -8,7 +8,7 @@ import { DomScrollableElement } from 'vs/base/browser/ui/scrollbar/scrollableEle
 import { ToggleMenuAction, ToolBar } from 'vs/base/browser/ui/toolbar/toolbar';
 import { IAction, Separator } from 'vs/base/common/actions';
 import { Emitter, Event } from 'vs/base/common/event';
-import { Disposable } from 'vs/base/common/lifecycle';
+import { Disposable, IDisposable } from 'vs/base/common/lifecycle';
 import { ScrollbarVisibility } from 'vs/base/common/scrollable';
 import { MenuEntryActionViewItem } from 'vs/platform/actions/browser/menuEntryActionViewItem';
 import { IMenu, IMenuService, MenuItemAction, SubmenuItemAction } from 'vs/platform/actions/common/actions';
@@ -24,8 +24,10 @@ import { NotebooKernelActionViewItem } from 'vs/workbench/contrib/notebook/brows
 import { ActionViewWithLabel } from 'vs/workbench/contrib/notebook/browser/view/cellParts/cellActionView';
 import { IEditorService } from 'vs/workbench/services/editor/common/editorService';
 import { IWorkbenchAssignmentService } from 'vs/workbench/services/assignment/common/assignmentService';
-import { NotebookOptions } from 'vs/workbench/contrib/notebook/common/notebookOptions';
+import { NotebookOptions } from 'vs/workbench/contrib/notebook/browser/notebookOptions';
 import { IActionViewItemProvider } from 'vs/base/browser/ui/actionbar/actionbar';
+import { disposableTimeout } from 'vs/base/common/async';
+import { ActionViewItem } from 'vs/base/browser/ui/actionbar/actionViewItems';
 
 interface IActionModel {
 	action: IAction;
@@ -59,18 +61,13 @@ class FixedLabelStrategy implements IActionLayoutStrategy {
 
 	}
 
-	actionProvider(action: IAction) {
+	actionProvider(action: IAction): ActionViewItem | undefined {
 		if (action.id === SELECT_KERNEL_ID) {
 			// 	// this is being disposed by the consumer
 			return this.instantiationService.createInstance(NotebooKernelActionViewItem, action, this.notebookEditor);
 		}
 
-		const a = this.editorToolbar.primaryActions.find(a => a.action.id === action.id);
-		if (a && a.renderLabel) {
-			return action instanceof MenuItemAction ? this.instantiationService.createInstance(ActionViewWithLabel, action, undefined) : undefined;
-		} else {
-			return action instanceof MenuItemAction ? this.instantiationService.createInstance(MenuEntryActionViewItem, action, undefined) : undefined;
-		}
+		return action instanceof MenuItemAction ? this.instantiationService.createInstance(ActionViewWithLabel, action, undefined) : undefined;
 	}
 
 	protected _calculateFixedActions(leftToolbarContainerMaxWidth: number) {
@@ -141,7 +138,8 @@ class DynamicLabelStrategy implements IActionLayoutStrategy {
 		}
 
 		const a = this.editorToolbar.primaryActions.find(a => a.action.id === action.id);
-		if (a && a.renderLabel) {
+		if (!a || a.renderLabel) {
+			// render new action with label to get a correct full width.
 			return action instanceof MenuItemAction ? this.instantiationService.createInstance(ActionViewWithLabel, action, undefined) : undefined;
 		} else {
 			return action instanceof MenuItemAction ? this.instantiationService.createInstance(MenuEntryActionViewItem, action, undefined) : undefined;
@@ -276,8 +274,15 @@ export class NotebookEditorToolbar extends Disposable {
 	private _strategy!: IActionLayoutStrategy;
 	private _renderLabel: RenderLabel = RenderLabel.Always;
 
-	private readonly _onDidChangeState = this._register(new Emitter<void>());
-	onDidChangeState: Event<void> = this._onDidChangeState.event;
+	private _visible: boolean = false;
+	set visible(visible: boolean) {
+		if (this._visible !== visible) {
+			this._visible = visible;
+			this._onDidChangeVisibility.fire(visible);
+		}
+	}
+	private readonly _onDidChangeVisibility = this._register(new Emitter<boolean>());
+	onDidChangeVisibility: Event<boolean> = this._onDidChangeVisibility.event;
 
 	get useGlobalToolbar(): boolean {
 		return this._useGlobalToolbar;
@@ -285,6 +290,8 @@ export class NotebookEditorToolbar extends Disposable {
 
 	private _dimension: DOM.Dimension | null = null;
 
+	private _deferredActionUpdate: IDisposable | undefined;
+
 	constructor(
 		readonly notebookEditor: INotebookEditorDelegate,
 		readonly contextKeyService: IContextKeyService,
@@ -375,8 +382,8 @@ export class NotebookEditorToolbar extends Disposable {
 
 		this._notebookLeftToolbar = new ToolBar(this._notebookTopLeftToolbarContainer, this.contextMenuService, {
 			getKeyBinding: action => this.keybindingService.lookupKeybinding(action.id),
-			actionViewItemProvider: (action) => {
-				return this._strategy.actionProvider(action);
+			actionViewItemProvider: (action, options) => {
+				return this._strategy.actionProvider(action, options);
 			},
 			renderDropdownAsChildElement: true
 		});
@@ -401,7 +408,9 @@ export class NotebookEditorToolbar extends Disposable {
 				return;
 			}
 
-			this._showNotebookActionsinEditorToolbar();
+			if (this.notebookEditor.isVisible) {
+				this._showNotebookActionsinEditorToolbar();
+			}
 		}));
 
 		this._register(this._notebookLeftToolbar.onDidChangeDropdownVisibility(visible => {
@@ -431,7 +440,9 @@ export class NotebookEditorToolbar extends Disposable {
 				this._notebookLeftToolbar.dispose();
 				this._notebookLeftToolbar = new ToolBar(this._notebookTopLeftToolbarContainer, this.contextMenuService, {
 					getKeyBinding: action => this.keybindingService.lookupKeybinding(action.id),
-					actionViewItemProvider: actionProvider,
+					actionViewItemProvider: (action, options) => {
+						return this._strategy.actionProvider(action, options);
+					},
 					renderDropdownAsChildElement: true
 				});
 				this._register(this._notebookLeftToolbar);
@@ -486,19 +497,30 @@ export class NotebookEditorToolbar extends Disposable {
 	private _showNotebookActionsinEditorToolbar() {
 		// when there is no view model, just ignore.
 		if (!this.notebookEditor.hasModel()) {
+			this._deferredActionUpdate?.dispose();
+			this._deferredActionUpdate = undefined;
+			this.visible = false;
+			return;
+		}
+
+		if (this._deferredActionUpdate) {
 			return;
 		}
 
 		if (!this._useGlobalToolbar) {
 			this.domNode.style.display = 'none';
+			this._deferredActionUpdate = undefined;
+			this.visible = false;
 		} else {
-			this._setNotebookActions();
+			this._deferredActionUpdate = disposableTimeout(async () => {
+				await this._setNotebookActions();
+				this.visible = true;
+				this._deferredActionUpdate = undefined;
+			}, 50);
 		}
-
-		this._onDidChangeState.fire();
 	}
 
-	private _setNotebookActions() {
+	private async _setNotebookActions() {
 		const groups = this._notebookGlobalActionsMenu.getActions({ shouldForwardArgs: true, renderShortTitle: true });
 		this.domNode.style.display = 'flex';
 		const primaryLeftGroups = groups.filter(group => /^navigation/.test(group[0]));
@@ -560,10 +582,10 @@ export class NotebookEditorToolbar extends Disposable {
 	private _canBeVisible(width: number) {
 		let w = 0;
 		for (let i = 0; i < this._primaryActions.length; i++) {
-			w += this._primaryActions[i].size + 8;
+			w += this._primaryActions[i].size + ACTION_PADDING;
 		}
 
-		return w <= width;
+		return w - ACTION_PADDING <= width;
 	}
 
 	private _computeSizes() {
@@ -614,6 +636,8 @@ export class NotebookEditorToolbar extends Disposable {
 		this._notebookRightToolbar.dispose();
 		this._notebookLeftToolbar = null!;
 		this._notebookRightToolbar = null!;
+		this._deferredActionUpdate?.dispose();
+		this._deferredActionUpdate = undefined;
 
 		super.dispose();
 	}
