@@ -93,7 +93,7 @@ export class NotebookCellTextModel extends Disposable implements ICell {
 
 		if (this._textModel) {
 			const languageId = this._languageService.createById(newLanguageId);
-			this._textModel.setMode(languageId.languageId);
+			this._textModel.setLanguage(languageId.languageId);
 		}
 
 		if (this._language === newLanguage) {
@@ -280,23 +280,25 @@ export class NotebookCellTextModel extends Disposable implements ICell {
 	}
 
 	spliceNotebookCellOutputs(splice: NotebookCellOutputsSplice): void {
-		this.outputs.splice(splice.start, splice.deleteCount, ...splice.newOutputs);
-		this._onDidChangeOutputs.fire(splice);
+		if (splice.deleteCount > 0 && splice.newOutputs.length > 0) {
+			const commonLen = Math.min(splice.deleteCount, splice.newOutputs.length);
+			// update
+			for (let i = 0; i < commonLen; i++) {
+				const currentOutput = this.outputs[splice.start + i];
+				const newOutput = splice.newOutputs[i];
+
+				this.replaceOutput(currentOutput.outputId, newOutput);
+			}
+
+			this.outputs.splice(splice.start + commonLen, splice.deleteCount - commonLen, ...splice.newOutputs.slice(commonLen));
+			this._onDidChangeOutputs.fire({ start: splice.start + commonLen, deleteCount: splice.deleteCount - commonLen, newOutputs: splice.newOutputs.slice(commonLen) });
+		} else {
+			this.outputs.splice(splice.start, splice.deleteCount, ...splice.newOutputs);
+			this._onDidChangeOutputs.fire(splice);
+		}
 	}
 
-	changeOutputItems(outputId: string, append: boolean, items: IOutputItemDto[]): boolean {
-		const outputIndex = this.outputs.findIndex(output => output.outputId === outputId);
-
-		if (outputIndex < 0) {
-			return false;
-		}
-
-		const output = this.outputs[outputIndex];
-		if (append) {
-			output.appendData(items);
-		} else {
-			output.replaceData(items);
-		}
+	private _optimizeOutputItems(output: ICellOutput) {
 		if (output.outputs.length > 1 && output.outputs.every(item => isTextStreamMime(item.mime))) {
 			// Look for the mimes in the items, and keep track of their order.
 			// Merge the streams into one output item, per mime type.
@@ -322,7 +324,37 @@ export class NotebookCellTextModel extends Disposable implements ICell {
 				});
 			});
 		}
+	}
 
+	replaceOutput(outputId: string, newOutputItem: ICellOutput) {
+		const outputIndex = this.outputs.findIndex(output => output.outputId === outputId);
+
+		if (outputIndex < 0) {
+			return false;
+		}
+
+		const output = this.outputs[outputIndex];
+		output.replaceData(newOutputItem);
+		this._optimizeOutputItems(output);
+		this._onDidChangeOutputItems.fire();
+		return true;
+	}
+
+	changeOutputItems(outputId: string, append: boolean, items: IOutputItemDto[]): boolean {
+		const outputIndex = this.outputs.findIndex(output => output.outputId === outputId);
+
+		if (outputIndex < 0) {
+			return false;
+		}
+
+		const output = this.outputs[outputIndex];
+		if (append) {
+			output.appendData(items);
+		} else {
+			output.replaceData({ outputId: outputId, outputs: items, metadata: output.metadata });
+		}
+
+		this._optimizeOutputItems(output);
 		this._onDidChangeOutputItems.fire();
 		return true;
 	}
@@ -403,7 +435,10 @@ export class NotebookCellTextModel extends Disposable implements ICell {
 			return false;
 		}
 
-		if (this._source !== b.source) {
+		// Once we attach the cell text buffer to an editor, the source of truth is the text buffer instead of the original source
+		if (this._textBuffer && this.getValue() !== b.source) {
+			return false;
+		} else if (this._source !== b.source) {
 			return false;
 		}
 
@@ -431,8 +466,7 @@ export function cloneNotebookCellTextModel(cell: NotebookCellTextModel) {
 			outputs: output.outputs,
 			/* paste should generate new outputId */ outputId: UUID.generateUuid()
 		})),
-		metadata: { ...cell.metadata },
-		// Don't include internalMetadata, ie execution state, this is not to be shared
+		metadata: {}
 	};
 }
 
