@@ -80,7 +80,6 @@ import { IPatternInfo, ISearchComplete, ISearchConfiguration, ISearchConfigurati
 import { TextSearchCompleteMessage } from 'vs/workbench/services/search/common/searchExtTypes';
 import { ITextFileService } from 'vs/workbench/services/textfile/common/textfiles';
 import { INotebookService } from 'vs/workbench/contrib/notebook/common/notebookService';
-import { NotebookFindContrib } from 'vs/workbench/contrib/notebook/browser/contrib/find/notebookFindWidget';
 import { ILogService } from 'vs/platform/log/common/log';
 
 const $ = dom.$;
@@ -240,7 +239,7 @@ export class SearchView extends ViewPane {
 		this.viewModel = this._register(this.searchWorkbenchService.searchModel);
 		this.queryBuilder = this.instantiationService.createInstance(QueryBuilder);
 		this.memento = new Memento(this.id, storageService);
-		this.viewletState = this.memento.getMemento(StorageScope.WORKSPACE, StorageTarget.USER);
+		this.viewletState = this.memento.getMemento(StorageScope.WORKSPACE, StorageTarget.MACHINE);
 
 		this._register(this.fileService.onDidFilesChange(e => this.onFilesChanged(e)));
 		this._register(this.textFileService.untitled.onWillDispose(model => this.onUntitledDidDispose(model.resource)));
@@ -466,6 +465,7 @@ export class SearchView extends ViewPane {
 		const preserveCase = this.viewletState['query.preserveCase'] === true;
 
 		const isInNotebookMarkdownInput = this.viewletState['query.isInNotebookMarkdownInput'] ?? true;
+		const isInNotebookMarkdownPreview = this.viewletState['query.isInNotebookMarkdownPreview'] ?? true;
 		const isInNotebookCellInput = this.viewletState['query.isInNotebookCellInput'] ?? true;
 		const isInNotebookCellOutput = this.viewletState['query.isInNotebookCellOutput'] ?? true;
 
@@ -483,6 +483,7 @@ export class SearchView extends ViewPane {
 			toggleStyles: defaultToggleStyles,
 			notebookOptions: {
 				isInNotebookMarkdownInput,
+				isInNotebookMarkdownPreview,
 				isInNotebookCellInput,
 				isInNotebookCellOutput,
 			}
@@ -851,8 +852,14 @@ export class SearchView extends ViewPane {
 				this.lastFocusState = 'tree';
 			}
 
-			// we don't need to check experimental flag here because NotebookMatches only exist when the flag is enabled
-			const editable = (!(focus instanceof MatchInNotebook)) || !focus.isWebviewMatch();
+			let editable = false;
+			if (focus instanceof MatchInNotebook) {
+				editable = !focus.isWebviewMatch();
+			} else if (focus instanceof FileMatch) {
+				editable = !focus.hasOnlyReadOnlyMatches();
+			} else if (focus instanceof FolderMatch) {
+				editable = !focus.hasOnlyReadOnlyMatches();
+			}
 			this.isEditableItem.set(editable);
 		}));
 
@@ -1410,6 +1417,7 @@ export class SearchView extends ViewPane {
 
 		const isRegex = this.searchWidget.searchInput.getRegex();
 		const isInNotebookMarkdownInput = this.searchWidget.getNotebookFilters().markupInput;
+		const isInNotebookMarkdownPreview = this.searchWidget.getNotebookFilters().markupPreview;
 		const isInNotebookCellInput = this.searchWidget.getNotebookFilters().codeInput;
 		const isInNotebookCellOutput = this.searchWidget.getNotebookFilters().codeOutput;
 
@@ -1434,6 +1442,7 @@ export class SearchView extends ViewPane {
 			isWordMatch: isWholeWords,
 			notebookInfo: {
 				isInNotebookMarkdownInput,
+				isInNotebookMarkdownPreview,
 				isInNotebookCellInput,
 				isInNotebookCellOutput
 			}
@@ -1811,8 +1820,6 @@ export class SearchView extends ViewPane {
 	private shouldOpenInNotebookEditor(match: Match, uri: URI): boolean {
 		// Untitled files will return a false positive for getContributedNotebookTypes.
 		// Since untitled files are already open, then untitled notebooks should return NotebookMatch results.
-
-		// notebookMatch are only created when search.experimental.notebookSearch is enabled, so this should never return true if experimental flag is disabled.
 		return match instanceof MatchInNotebook || (uri.scheme !== network.Schemas.untitled && this.notebookService.getContributedNotebookTypes(uri).length > 0);
 	}
 
@@ -1857,40 +1864,32 @@ export class SearchView extends ViewPane {
 
 		if (editor instanceof NotebookEditor) {
 			const elemParent = element.parent() as FileMatch;
-			const experimentalNotebooksEnabled = this.configurationService.getValue<ISearchConfigurationProperties>('search').experimental.notebookSearch;
-			if (experimentalNotebooksEnabled) {
-				if (element instanceof Match) {
-					if (element instanceof MatchInNotebook) {
-						element.parent().showMatch(element);
-					} else {
-						const editorWidget = editor.getControl();
-						if (editorWidget) {
-							// Ensure that the editor widget is binded. If if is, then this should return immediately.
-							// Otherwise, it will bind the widget.
-							await elemParent.bindNotebookEditorWidget(editorWidget);
-							await elemParent.updateMatchesForEditorWidget();
+			if (element instanceof Match) {
+				if (element instanceof MatchInNotebook) {
+					element.parent().showMatch(element);
+				} else {
+					const editorWidget = editor.getControl();
+					if (editorWidget) {
+						// Ensure that the editor widget is binded. If if is, then this should return immediately.
+						// Otherwise, it will bind the widget.
+						elemParent.bindNotebookEditorWidget(editorWidget);
+						await elemParent.updateMatchesForEditorWidget();
 
-							const matchIndex = oldParentMatches.findIndex(e => e.id() === element.id());
-							const matches = element.parent().matches();
-							const match = matchIndex >= matches.length ? matches[matches.length - 1] : matches[matchIndex];
+						const matchIndex = oldParentMatches.findIndex(e => e.id() === element.id());
+						const matches = element.parent().matches();
+						const match = matchIndex >= matches.length ? matches[matches.length - 1] : matches[matchIndex];
 
-							if (match instanceof MatchInNotebook) {
-								elemParent.showMatch(match);
-							}
-
-							if (!this.tree.getFocus().includes(match) || !this.tree.getSelection().includes(match)) {
-								this.tree.setSelection([match], getSelectionKeyboardEvent());
-							}
+						if (match instanceof MatchInNotebook) {
+							elemParent.showMatch(match);
 						}
 
+						if (!this.tree.getFocus().includes(match) || !this.tree.getSelection().includes(match)) {
+							this.tree.setSelection([match], getSelectionKeyboardEvent());
+							this.tree.setFocus([match]);
+						}
 					}
 				}
-			} else {
-				const controller = editor.getControl()?.getContribution<NotebookFindContrib>(NotebookFindContrib.id);
-				const matchIndex = element instanceof Match ? element.parent().matches().findIndex(e => e.id() === element.id()) : undefined;
-				controller?.show(this.searchWidget.searchInput?.getValue(), { matchIndex, focus: false });
 			}
-
 		}
 	}
 
