@@ -203,7 +203,7 @@ suite('Search - Viewlet', () => {
 		instantiationService.stub(IThemeService, new TestThemeService());
 
 		const config = new TestConfigurationService();
-		config.setUserConfiguration('search', { searchOnType: true, experimental: { notebookSearch: false } });
+		config.setUserConfiguration('search', { searchOnType: true });
 		instantiationService.stub(IConfigurationService, config);
 
 		return instantiationService.createInstance(ModelService);
