@@ -5,15 +5,16 @@
 
 import { ok } from 'assert';
 import { Emitter, Event } from 'vs/base/common/event';
+import { AudioCue, IAudioCueService } from 'vs/platform/audioCues/browser/audioCueService';
 import { TestConfigurationService } from 'vs/platform/configuration/test/common/testConfigurationService';
 import { TestInstantiationService } from 'vs/platform/instantiation/test/common/instantiationServiceMock';
-import { AudioCue, IAudioCueService } from 'vs/workbench/contrib/audioCues/browser/audioCueService';
 import { ACTIVE_TASK_STATUS, FAILED_TASK_STATUS, SUCCEEDED_TASK_STATUS, TaskTerminalStatus } from 'vs/workbench/contrib/tasks/browser/taskTerminalStatus';
 import { AbstractProblemCollector } from 'vs/workbench/contrib/tasks/common/problemCollectors';
 import { CommonTask, ITaskEvent, TaskEventKind, TaskRunType } from 'vs/workbench/contrib/tasks/common/tasks';
 import { ITaskService, Task } from 'vs/workbench/contrib/tasks/common/taskService';
 import { ITerminalInstance } from 'vs/workbench/contrib/terminal/browser/terminal';
-import { ITerminalStatus, ITerminalStatusList, TerminalStatusList } from 'vs/workbench/contrib/terminal/browser/terminalStatusList';
+import { ITerminalStatusList, TerminalStatusList } from 'vs/workbench/contrib/terminal/browser/terminalStatusList';
+import { ITerminalStatus } from 'vs/workbench/contrib/terminal/common/terminal';
 
 class TestTaskService implements Partial<ITaskService> {
 	private readonly _onDidStateChange: Emitter<ITaskEvent> = new Emitter();
@@ -36,6 +37,11 @@ class TestTerminal implements Partial<ITerminalInstance> {
 }
 
 class TestTask extends CommonTask {
+
+	constructor() {
+		super('test', undefined, undefined, {}, {}, { kind: '', label: '' });
+	}
+
 	protected getFolderId(): string | undefined {
 		throw new Error('Method not implemented.');
 	}
@@ -66,9 +72,9 @@ suite('Task Terminal Status', () => {
 		taskService = new TestTaskService();
 		audioCueService = new TestAudioCueService();
 		taskTerminalStatus = new TaskTerminalStatus(taskService as any, audioCueService as any);
-		testTerminal = instantiationService.createInstance(TestTerminal);
-		testTask = instantiationService.createInstance(TestTask);
-		problemCollector = instantiationService.createInstance(TestProblemCollector);
+		testTerminal = instantiationService.createInstance(TestTerminal) as any;
+		testTask = instantiationService.createInstance(TestTask) as unknown as Task;
+		problemCollector = instantiationService.createInstance(TestProblemCollector) as any;
 	});
 	test('Should add failed status when there is an exit code on task end', async () => {
 		taskTerminalStatus.addTerminal(testTask, testTerminal, problemCollector);
