@@ -66,7 +66,7 @@ class ProblemReporter implements IProblemReporter {
 	}
 }
 
-class ConfiguationBuilder {
+class ConfigurationBuilder {
 
 	public result: Tasks.Task[];
 	private builders: CustomTaskBuilder[];
@@ -196,7 +196,7 @@ class CustomTaskBuilder {
 	public result: Tasks.CustomTask;
 	private commandBuilder: CommandConfigurationBuilder;
 
-	constructor(public parent: ConfiguationBuilder, name: string, command: string) {
+	constructor(public parent: ConfigurationBuilder, name: string, command: string) {
 		this.commandBuilder = new CommandConfigurationBuilder(this, command);
 		this.result = new Tasks.CustomTask(
 			name,
@@ -386,7 +386,7 @@ function testDefaultProblemMatcher(external: IExternalTaskRunnerConfiguration, r
 	assert.strictEqual(task.configurationProperties.problemMatchers!.length, resolved);
 }
 
-function testConfiguration(external: IExternalTaskRunnerConfiguration, builder: ConfiguationBuilder): void {
+function testConfiguration(external: IExternalTaskRunnerConfiguration, builder: ConfigurationBuilder): void {
 	builder.done();
 	const reporter = new ProblemReporter();
 	const result = parse(workspaceFolder, workspace, Platform.platform, external, reporter, TaskConfigSource.TasksJson, new TasksMockContextKeyService());
@@ -599,7 +599,7 @@ function assertProblemPattern(actual: IProblemPattern, expected: IProblemPattern
 
 suite('Tasks version 0.1.0', () => {
 	test('tasks: all default', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
 			command().suppressTaskName(true);
@@ -611,7 +611,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global isShellCommand', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
 			command().suppressTaskName(true).
@@ -626,7 +626,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global show output silent', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -643,7 +643,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global promptOnClose default', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
 			command().suppressTaskName(true);
@@ -658,7 +658,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global promptOnClose', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
 			promptOnClose(false).
@@ -674,7 +674,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global promptOnClose default watching', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
 			isBackground(true).
@@ -691,7 +691,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global show output never', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -708,7 +708,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global echo Command', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -726,7 +726,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: global args', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -745,7 +745,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: options - cwd', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -766,7 +766,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: options - env', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -788,7 +788,7 @@ suite('Tasks version 0.1.0', () => {
 
 	test('tasks: os windows', () => {
 		const name: string = Platform.isWindows ? 'tsc.win' : 'tsc';
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task(name, name).
 			group(Tasks.TaskGroup.Build).
@@ -805,7 +805,7 @@ suite('Tasks version 0.1.0', () => {
 
 	test('tasks: os windows & global isShellCommand', () => {
 		const name: string = Platform.isWindows ? 'tsc.win' : 'tsc';
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task(name, name).
 			group(Tasks.TaskGroup.Build).
@@ -824,7 +824,7 @@ suite('Tasks version 0.1.0', () => {
 
 	test('tasks: os mac', () => {
 		const name: string = Platform.isMacintosh ? 'tsc.osx' : 'tsc';
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task(name, name).
 			group(Tasks.TaskGroup.Build).
@@ -841,7 +841,7 @@ suite('Tasks version 0.1.0', () => {
 
 	test('tasks: os linux', () => {
 		const name: string = Platform.isLinux ? 'tsc.linux' : 'tsc';
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task(name, name).
 			group(Tasks.TaskGroup.Build).
@@ -857,7 +857,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: overwrite showOutput', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -875,7 +875,7 @@ suite('Tasks version 0.1.0', () => {
 	});
 
 	test('tasks: overwrite echo Command', () => {
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.
 			task('tsc', 'tsc').
 			group(Tasks.TaskGroup.Build).
@@ -921,7 +921,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').command().args(['$name']);
 		testConfiguration(external, builder);
 	});
@@ -937,7 +937,7 @@ suite('Tasks version 0.1.0', () => {
 				} as ICustomTask
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').group(Tasks.TaskGroup.Build).command().args(['$name']);
 		testConfiguration(external, builder);
 	});
@@ -952,7 +952,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('build', 'tsc').group(Tasks.TaskGroup.Build).command().args(['$name']);
 		testConfiguration(external, builder);
 	});
@@ -968,7 +968,7 @@ suite('Tasks version 0.1.0', () => {
 				} as ICustomTask
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').group(Tasks.TaskGroup.Test).command().args(['$name']);
 		testConfiguration(external, builder);
 	});
@@ -983,7 +983,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('test', 'tsc').group(Tasks.TaskGroup.Test).command().args(['$name']);
 		testConfiguration(external, builder);
 	});
@@ -1002,7 +1002,7 @@ suite('Tasks version 0.1.0', () => {
 				} as ICustomTask
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('test', 'tsc').
 			group(Tasks.TaskGroup.Test).
 			isBackground(true).
@@ -1026,7 +1026,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('test', 'tsc').
 			group(Tasks.TaskGroup.Test).
 			command().args(['$name']).presentation().
@@ -1050,7 +1050,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().args(['$name']).parent.
 			problemMatcher().pattern(/abc/);
@@ -1072,7 +1072,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().args(['$name']).parent.
 			problemMatcher().pattern(/.*/);
@@ -1098,7 +1098,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().args(['$name']).parent.
 			problemMatcher().
@@ -1127,7 +1127,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().args(['$name']).parent.
 			problemMatcher().
@@ -1157,7 +1157,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().args(['$name']).parent.
 			problemMatcher().
@@ -1188,7 +1188,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().args(['$name']).parent.
 			problemMatcher().
@@ -1208,7 +1208,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			promptOnClose(true).
 			command().args(['$name']);
@@ -1226,7 +1226,7 @@ suite('Tasks version 0.1.0', () => {
 				} as ICustomTask
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			isBackground(true).promptOnClose(false).
 			command().args(['$name']);
@@ -1244,7 +1244,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			promptOnClose(false).
 			command().args(['$name']);
@@ -1262,7 +1262,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().
 			taskSelector('/t:').
@@ -1282,7 +1282,7 @@ suite('Tasks version 0.1.0', () => {
 				} as ICustomTask
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().suppressTaskName(true);
 		testConfiguration(external, builder);
@@ -1299,7 +1299,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskName', 'tsc').
 			command().suppressTaskName(true);
 		testConfiguration(external, builder);
@@ -1318,7 +1318,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', 'tsc').
 			command().args(['$name']);
 		builder.task('taskNameTwo', 'tsc').
@@ -1336,7 +1336,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', 'tsc').command().suppressTaskName(true);
 		testConfiguration(external, builder);
 	});
@@ -1355,7 +1355,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', 'tsc').command().suppressTaskName(true);
 		builder.task('taskNameTwo', 'dir').command().suppressTaskName(true);
 		testConfiguration(external, builder);
@@ -1379,7 +1379,7 @@ suite('Tasks version 0.1.0', () => {
 				} as ICustomTask
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', 'tsc').command().suppressTaskName(true).
 			runtime(Tasks.RuntimeType.Shell).args(['arg']).options({ cwd: 'cwd', env: { env: 'env' } });
 		testConfiguration(external, builder);
@@ -1399,7 +1399,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', name).command().suppressTaskName(true);
 		testConfiguration(external, builder);
 	});
@@ -1419,7 +1419,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('tsc', 'tsc').command().suppressTaskName(true).args(args);
 		testConfiguration(external, builder);
 	});
@@ -1439,7 +1439,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('tsc', 'tsc').command().suppressTaskName(true).args(args);
 		testConfiguration(external, builder);
 	});
@@ -1455,7 +1455,7 @@ suite('Tasks version 0.1.0', () => {
 				} as ICustomTask
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', 'tsc').command().runtime(Tasks.RuntimeType.Shell).args(['$name']);
 		testConfiguration(external, builder);
 	});
@@ -1472,7 +1472,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', 'tsc').command().args(['global', '$name', 'local']);
 		testConfiguration(external, builder);
 	});
@@ -1490,7 +1490,7 @@ suite('Tasks version 0.1.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('taskNameOne', 'tsc').command().taskSelector('/t:').args(['global', '/t:taskNameOne', 'local']);
 		testConfiguration(external, builder);
 	});
@@ -1509,7 +1509,7 @@ suite('Tasks version 2.0.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('dir', 'dir').
 			group(Tasks.TaskGroup.Build).
 			command().suppressTaskName(true).
@@ -1524,7 +1524,7 @@ suite('Tasks version 2.0.0', () => {
 			type: 'shell',
 			group: 'none'
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('dir', 'dir').
 			command().suppressTaskName(true).
 			runtime(Tasks.RuntimeType.Shell).
@@ -1538,7 +1538,7 @@ suite('Tasks version 2.0.0', () => {
 			type: 'shell',
 			group: 'build'
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('dir', 'dir').
 			group(Tasks.TaskGroup.Build).
 			command().suppressTaskName(true).
@@ -1553,7 +1553,7 @@ suite('Tasks version 2.0.0', () => {
 			type: 'shell',
 			group: { kind: 'build', isDefault: true }
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		const taskGroup = Tasks.TaskGroup.Build;
 		taskGroup.isDefault = true;
 		builder.task('dir', 'dir').
@@ -1575,7 +1575,7 @@ suite('Tasks version 2.0.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('dir', 'dir').
 			command().suppressTaskName(true).
 			runtime(Tasks.RuntimeType.Shell).
@@ -1594,7 +1594,7 @@ suite('Tasks version 2.0.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('dir', 'dir').
 			group(Tasks.TaskGroup.Build).
 			command().suppressTaskName(true).
@@ -1614,7 +1614,7 @@ suite('Tasks version 2.0.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		const taskGroup = Tasks.TaskGroup.Build;
 		taskGroup.isDefault = true;
 		builder.task('dir', 'dir').
@@ -1653,7 +1653,7 @@ suite('Tasks version 2.0.0', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		if (Platform.isWindows) {
 			builder.task('echo', 'echo').
 				command().suppressTaskName(true).args(['windows']).
@@ -1722,7 +1722,7 @@ suite('Bugs / regression tests', () => {
 				]
 			}
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		if (Platform.isWindows) {
 			builder.task('composeForDebug', 'powershell').
 				command().suppressTaskName(true).
@@ -1757,7 +1757,7 @@ suite('Bugs / regression tests', () => {
 				}
 			]
 		};
-		const builder = new ConfiguationBuilder();
+		const builder = new ConfigurationBuilder();
 		builder.task('build', 'bash').
 			group(Tasks.TaskGroup.Build).
 			command().suppressTaskName(true).
