@@ -4,7 +4,7 @@
  *--------------------------------------------------------------------------------------------*/
 
 import { deepStrictEqual } from 'assert';
-import { TestExtensionService, TestStorageService } from 'vs/workbench/test/common/workbenchTestServices';
+import { TestExtensionService, TestHistoryService, TestStorageService } from 'vs/workbench/test/common/workbenchTestServices';
 import { EnvironmentVariableService } from 'vs/workbench/contrib/terminal/common/environmentVariableService';
 import { EnvironmentVariableMutatorType, IEnvironmentVariableMutator } from 'vs/platform/terminal/common/environmentVariable';
 import { IStorageService } from 'vs/platform/storage/common/storage';
@@ -12,6 +12,8 @@ import { TestInstantiationService } from 'vs/platform/instantiation/test/common/
 import { IExtensionService } from 'vs/workbench/services/extensions/common/extensions';
 import { Emitter } from 'vs/base/common/event';
 import { IProcessEnvironment } from 'vs/base/common/platform';
+import { IHistoryService } from 'vs/workbench/services/history/common/history';
+import { URI } from 'vs/base/common/uri';
 
 class TestEnvironmentVariableService extends EnvironmentVariableService {
 	persistCollections(): void { this._persistCollections(); }
@@ -22,6 +24,7 @@ suite('EnvironmentVariable - EnvironmentVariableService', () => {
 	let instantiationService: TestInstantiationService;
 	let environmentVariableService: TestEnvironmentVariableService;
 	let storageService: TestStorageService;
+	let historyService: TestHistoryService;
 	let changeExtensionsEvent: Emitter<void>;
 
 	setup(() => {
@@ -30,6 +33,7 @@ suite('EnvironmentVariable - EnvironmentVariableService', () => {
 		instantiationService = new TestInstantiationService();
 		instantiationService.stub(IExtensionService, TestExtensionService);
 		storageService = new TestStorageService();
+		historyService = new TestHistoryService();
 		instantiationService.stub(IStorageService, storageService);
 		instantiationService.stub(IExtensionService, TestExtensionService);
 		instantiationService.stub(IExtensionService, 'onDidChangeExtensions', changeExtensionsEvent.event);
@@ -38,29 +42,30 @@ suite('EnvironmentVariable - EnvironmentVariableService', () => {
 			{ identifier: { value: 'ext2' } },
 			{ identifier: { value: 'ext3' } }
 		]);
+		instantiationService.stub(IHistoryService, historyService);
 
 		environmentVariableService = instantiationService.createInstance(TestEnvironmentVariableService);
 	});
 
 	test('should persist collections to the storage service and be able to restore from them', () => {
 		const collection = new Map<string, IEnvironmentVariableMutator>();
-		collection.set('A', { value: 'a', type: EnvironmentVariableMutatorType.Replace });
-		collection.set('B', { value: 'b', type: EnvironmentVariableMutatorType.Append });
-		collection.set('C', { value: 'c', type: EnvironmentVariableMutatorType.Prepend });
+		collection.set('A-key', { value: 'a', type: EnvironmentVariableMutatorType.Replace, variable: 'A' });
+		collection.set('B-key', { value: 'b', type: EnvironmentVariableMutatorType.Append, variable: 'B' });
+		collection.set('C-key', { value: 'c', type: EnvironmentVariableMutatorType.Prepend, variable: 'C' });
 		environmentVariableService.set('ext1', { map: collection, persistent: true });
-		deepStrictEqual([...environmentVariableService.mergedCollection.map.entries()], [
-			['A', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Replace, value: 'a' }]],
-			['B', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: 'b' }]],
-			['C', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Prepend, value: 'c' }]]
+		deepStrictEqual([...environmentVariableService.mergedCollection.getVariableMap(undefined).entries()], [
+			['A', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Replace, value: 'a', variable: 'A' }]],
+			['B', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: 'b', variable: 'B' }]],
+			['C', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Prepend, value: 'c', variable: 'C' }]]
 		]);
 
 		// Persist with old service, create a new service with the same storage service to verify restore
 		environmentVariableService.persistCollections();
 		const service2: TestEnvironmentVariableService = instantiationService.createInstance(TestEnvironmentVariableService);
-		deepStrictEqual([...service2.mergedCollection.map.entries()], [
-			['A', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Replace, value: 'a' }]],
-			['B', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: 'b' }]],
-			['C', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Prepend, value: 'c' }]]
+		deepStrictEqual([...service2.mergedCollection.getVariableMap(undefined).entries()], [
+			['A', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Replace, value: 'a', variable: 'A' }]],
+			['B', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: 'b', variable: 'B' }]],
+			['C', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Prepend, value: 'c', variable: 'C' }]]
 		]);
 	});
 
@@ -69,21 +74,21 @@ suite('EnvironmentVariable - EnvironmentVariableService', () => {
 			const collection1 = new Map<string, IEnvironmentVariableMutator>();
 			const collection2 = new Map<string, IEnvironmentVariableMutator>();
 			const collection3 = new Map<string, IEnvironmentVariableMutator>();
-			collection1.set('A', { value: 'a1', type: EnvironmentVariableMutatorType.Append });
-			collection1.set('B', { value: 'b1', type: EnvironmentVariableMutatorType.Replace });
-			collection2.set('A', { value: 'a2', type: EnvironmentVariableMutatorType.Replace });
-			collection2.set('B', { value: 'b2', type: EnvironmentVariableMutatorType.Append });
-			collection3.set('A', { value: 'a3', type: EnvironmentVariableMutatorType.Prepend });
-			collection3.set('B', { value: 'b3', type: EnvironmentVariableMutatorType.Replace });
+			collection1.set('A-key', { value: 'a1', type: EnvironmentVariableMutatorType.Append, variable: 'A' });
+			collection1.set('B-key', { value: 'b1', type: EnvironmentVariableMutatorType.Replace, variable: 'B' });
+			collection2.set('A-key', { value: 'a2', type: EnvironmentVariableMutatorType.Replace, variable: 'A' });
+			collection2.set('B-key', { value: 'b2', type: EnvironmentVariableMutatorType.Append, variable: 'B' });
+			collection3.set('A-key', { value: 'a3', type: EnvironmentVariableMutatorType.Prepend, variable: 'A' });
+			collection3.set('B-key', { value: 'b3', type: EnvironmentVariableMutatorType.Replace, variable: 'B' });
 			environmentVariableService.set('ext1', { map: collection1, persistent: true });
 			environmentVariableService.set('ext2', { map: collection2, persistent: true });
 			environmentVariableService.set('ext3', { map: collection3, persistent: true });
-			deepStrictEqual([...environmentVariableService.mergedCollection.map.entries()], [
+			deepStrictEqual([...environmentVariableService.mergedCollection.getVariableMap(undefined).entries()], [
 				['A', [
-					{ extensionIdentifier: 'ext2', type: EnvironmentVariableMutatorType.Replace, value: 'a2' },
-					{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: 'a1' }
+					{ extensionIdentifier: 'ext2', type: EnvironmentVariableMutatorType.Replace, value: 'a2', variable: 'A' },
+					{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: 'a1', variable: 'A' }
 				]],
-				['B', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Replace, value: 'b1' }]]
+				['B', [{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Replace, value: 'b1', variable: 'B' }]]
 			]);
 		});
 
@@ -91,26 +96,53 @@ suite('EnvironmentVariable - EnvironmentVariableService', () => {
 			const collection1 = new Map<string, IEnvironmentVariableMutator>();
 			const collection2 = new Map<string, IEnvironmentVariableMutator>();
 			const collection3 = new Map<string, IEnvironmentVariableMutator>();
-			collection1.set('A', { value: ':a1', type: EnvironmentVariableMutatorType.Append });
-			collection2.set('A', { value: 'a2:', type: EnvironmentVariableMutatorType.Prepend });
-			collection3.set('A', { value: 'a3', type: EnvironmentVariableMutatorType.Replace });
+			collection1.set('A-key', { value: ':a1', type: EnvironmentVariableMutatorType.Append, variable: 'A' });
+			collection2.set('A-key', { value: 'a2:', type: EnvironmentVariableMutatorType.Prepend, variable: 'A' });
+			collection3.set('A-key', { value: 'a3', type: EnvironmentVariableMutatorType.Replace, variable: 'A' });
 			environmentVariableService.set('ext1', { map: collection1, persistent: true });
 			environmentVariableService.set('ext2', { map: collection2, persistent: true });
 			environmentVariableService.set('ext3', { map: collection3, persistent: true });
 
 			// The entries should be ordered in the order they are applied
-			deepStrictEqual([...environmentVariableService.mergedCollection.map.entries()], [
+			deepStrictEqual([...environmentVariableService.mergedCollection.getVariableMap(undefined).entries()], [
 				['A', [
-					{ extensionIdentifier: 'ext3', type: EnvironmentVariableMutatorType.Replace, value: 'a3' },
-					{ extensionIdentifier: 'ext2', type: EnvironmentVariableMutatorType.Prepend, value: 'a2:' },
-					{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: ':a1' }
+					{ extensionIdentifier: 'ext3', type: EnvironmentVariableMutatorType.Replace, value: 'a3', variable: 'A' },
+					{ extensionIdentifier: 'ext2', type: EnvironmentVariableMutatorType.Prepend, value: 'a2:', variable: 'A' },
+					{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: ':a1', variable: 'A' }
 				]]
 			]);
 
 			// Verify the entries get applied to the environment as expected
 			const env: IProcessEnvironment = { A: 'foo' };
-			await environmentVariableService.mergedCollection.applyToProcessEnvironment(env);
+			await environmentVariableService.mergedCollection.applyToProcessEnvironment(env, undefined);
 			deepStrictEqual(env, { A: 'a2:a3:a1' });
 		});
+
+		test('should correctly apply the workspace specific environment values from multiple extension contributions in the correct order', async () => {
+			const scope1 = { workspaceFolder: { uri: URI.file('workspace1'), name: 'workspace1', index: 0 } };
+			const scope2 = { workspaceFolder: { uri: URI.file('workspace2'), name: 'workspace2', index: 3 } };
+			const collection1 = new Map<string, IEnvironmentVariableMutator>();
+			const collection2 = new Map<string, IEnvironmentVariableMutator>();
+			const collection3 = new Map<string, IEnvironmentVariableMutator>();
+			collection1.set('A-key', { value: ':a1', type: EnvironmentVariableMutatorType.Append, scope: scope1, variable: 'A' });
+			collection2.set('A-key', { value: 'a2:', type: EnvironmentVariableMutatorType.Prepend, variable: 'A' });
+			collection3.set('A-key', { value: 'a3', type: EnvironmentVariableMutatorType.Replace, scope: scope2, variable: 'A' });
+			environmentVariableService.set('ext1', { map: collection1, persistent: true });
+			environmentVariableService.set('ext2', { map: collection2, persistent: true });
+			environmentVariableService.set('ext3', { map: collection3, persistent: true });
+
+			// The entries should be ordered in the order they are applied
+			deepStrictEqual([...environmentVariableService.mergedCollection.getVariableMap(scope1).entries()], [
+				['A', [
+					{ extensionIdentifier: 'ext2', type: EnvironmentVariableMutatorType.Prepend, value: 'a2:', variable: 'A' },
+					{ extensionIdentifier: 'ext1', type: EnvironmentVariableMutatorType.Append, value: ':a1', scope: scope1, variable: 'A' }
+				]]
+			]);
+
+			// Verify the entries get applied to the environment as expected
+			const env: IProcessEnvironment = { A: 'foo' };
+			await environmentVariableService.mergedCollection.applyToProcessEnvironment(env, scope1);
+			deepStrictEqual(env, { A: 'a2:foo:a1' });
+		});
 	});
 });
