@@ -77,13 +77,13 @@ import { IObservableValue, MutableObservableValue, staticObservableValue } from
 import { StoredValue } from 'vs/workbench/contrib/testing/common/storedValue';
 import { ITestExplorerFilterState } from 'vs/workbench/contrib/testing/common/testExplorerFilterState';
 import { ITestProfileService } from 'vs/workbench/contrib/testing/common/testProfileService';
-import { ITestResult, TestResultItemChange, TestResultItemChangeReason, maxCountPriority, resultItemParents } from 'vs/workbench/contrib/testing/common/testResult';
+import { ITestResult, LiveTestResult, TestResultItemChange, TestResultItemChangeReason, maxCountPriority, resultItemParents } from 'vs/workbench/contrib/testing/common/testResult';
 import { ITestResultService, ResultChangeEvent } from 'vs/workbench/contrib/testing/common/testResultService';
 import { ITestService } from 'vs/workbench/contrib/testing/common/testService';
 import { IRichLocation, ITestErrorMessage, ITestItem, ITestMessage, ITestRunTask, ITestTaskState, TestMessageType, TestResultItem, TestResultState, TestRunProfileBitset } from 'vs/workbench/contrib/testing/common/testTypes';
 import { TestingContextKeys } from 'vs/workbench/contrib/testing/common/testingContextKeys';
 import { IShowResultOptions, ITestingPeekOpener } from 'vs/workbench/contrib/testing/common/testingPeekOpener';
-import { isFailedState } from 'vs/workbench/contrib/testing/common/testingStates';
+import { cmpPriority, isFailedState } from 'vs/workbench/contrib/testing/common/testingStates';
 import { ParsedTestUri, TestUriType, buildTestUri, parseTestUri } from 'vs/workbench/contrib/testing/common/testingUri';
 import { IEditorService } from 'vs/workbench/services/editor/common/editorService';
 
@@ -115,16 +115,16 @@ class MessageSubject {
 	}
 }
 
-class ResultSubject {
+class TaskSubject {
 	public readonly outputUri: URI;
 	public readonly revealLocation: undefined;
 
-	constructor(public readonly resultId: string) {
-		this.outputUri = buildTestUri({ resultId, type: TestUriType.AllOutput });
+	constructor(public readonly resultId: string, public readonly taskIndex: number) {
+		this.outputUri = buildTestUri({ resultId, taskIndex, type: TestUriType.TaskOutput });
 	}
 }
 
-type InspectSubject = MessageSubject | ResultSubject;
+type InspectSubject = MessageSubject | TaskSubject;
 
 /** Iterates through every message in every result */
 function* allMessages(results: readonly ITestResult[]) {
@@ -251,7 +251,7 @@ export class TestingPeekOpener extends Disposable implements ITestingPeekOpener
 		}
 
 		const options = { pinned: false, revealIfOpened: true };
-		if (current instanceof ResultSubject) {
+		if (current instanceof TaskSubject) {
 			this.editorService.openEditor({ resource: current.outputUri, options });
 			return;
 		}
@@ -579,7 +579,7 @@ export class TestingOutputPeekController extends Disposable implements IEditorCo
 
 		let found = false;
 		for (const { messageIndex, taskIndex, result, test } of allMessages(this.testResults.results)) {
-			if (subject instanceof ResultSubject && result.id === subject.resultId) {
+			if (subject instanceof TaskSubject && result.id === subject.resultId) {
 				found = true; // open the first message found in the current result
 			}
 
@@ -609,7 +609,7 @@ export class TestingOutputPeekController extends Disposable implements IEditorCo
 
 		let previous: { messageIndex: number; taskIndex: number; result: ITestResult; test: TestResultItem } | undefined;
 		for (const m of allMessages(this.testResults.results)) {
-			if (subject instanceof ResultSubject) {
+			if (subject instanceof TaskSubject) {
 				if (m.result.id === subject.resultId) {
 					break;
 				}
@@ -672,8 +672,8 @@ export class TestingOutputPeekController extends Disposable implements IEditorCo
 			return undefined;
 		}
 
-		if (parts.type === TestUriType.AllOutput) {
-			return new ResultSubject(parts.resultId);
+		if (parts.type === TestUriType.TaskOutput) {
+			return new TaskSubject(parts.resultId, parts.taskIndex);
 		}
 
 		const { resultId, testExtId, taskIndex, messageIndex } = parts;
@@ -854,7 +854,7 @@ class TestResultsPeek extends PeekViewWidget {
 	 * Updates the test to be shown.
 	 */
 	public setModel(subject: InspectSubject): Promise<void> {
-		if (subject instanceof ResultSubject) {
+		if (subject instanceof TaskSubject) {
 			this.current = subject;
 			return this.showInPlace(subject);
 		}
@@ -932,17 +932,6 @@ export class TestResultsView extends ViewPane {
 		@ITestResultService private readonly resultService: ITestResultService,
 	) {
 		super(options, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService);
-
-		this._register(resultService.onResultsChanged(ev => {
-			if (!this.isVisible()) {
-				return;
-			}
-
-			if ('started' in ev) {
-				// allow the tree to update so that the item exists
-				queueMicrotask(() => this.content.reveal({ subject: new ResultSubject(ev.started.id), preserveFocus: true }));
-			}
-		}));
 	}
 
 	public get subject() {
@@ -955,8 +944,8 @@ export class TestResultsView extends ViewPane {
 		this.content.onDidRequestReveal(subject => this.content.reveal({ preserveFocus: true, subject }));
 
 		const [lastResult] = this.resultService.results;
-		if (lastResult) {
-			this.content.reveal({ preserveFocus: true, subject: new ResultSubject(lastResult.id) });
+		if (lastResult && lastResult.tasks.length) {
+			this.content.reveal({ preserveFocus: true, subject: new TaskSubject(lastResult.id, 0) });
 		}
 	}
 
@@ -1004,7 +993,7 @@ const diffEditorOptions: IDiffEditorConstructionOptions = {
 	renderSideBySide: true,
 	originalAriaLabel: localize('testingOutputExpected', 'Expected result'),
 	modifiedAriaLabel: localize('testingOutputActual', 'Actual result'),
-	diffAlgorithm: 'smart',
+	diffAlgorithm: 'advanced',
 };
 
 const isDiffable = (message: ITestMessage): message is ITestErrorMessage & { actualOutput: string; expectedOutput: string } =>
@@ -1282,6 +1271,7 @@ interface ITreeElement {
 	context: unknown;
 	id: string;
 	label: string;
+	onDidChange: Event<void>;
 	labelWithIcons?: readonly (HTMLSpanElement | string)[];
 	icon?: ThemeIcon;
 	description?: string;
@@ -1289,6 +1279,8 @@ interface ITreeElement {
 }
 
 class TestResultElement implements ITreeElement {
+	public readonly changeEmitter = new Emitter<void>();
+	public readonly onDidChange = this.changeEmitter.event;
 	public readonly type = 'result';
 	public readonly context = this.value.id;
 	public readonly id = this.value.id;
@@ -1306,20 +1298,33 @@ class TestResultElement implements ITreeElement {
 }
 
 class TestCaseElement implements ITreeElement {
+	public readonly changeEmitter = new Emitter<void>();
+	public readonly onDidChange = this.changeEmitter.event;
 	public readonly type = 'test';
 	public readonly context = this.test.item.extId;
 	public readonly id = `${this.results.id}/${this.test.item.extId}`;
-	public readonly label = this.test.item.label;
-	public readonly labelWithIcons = renderLabelWithIcons(this.label);
 	public readonly description?: string;
 
+	public get state() {
+		return this.test.tasks[this.taskIndex].state;
+	}
+
+	public get label() {
+		return this.test.item.label;
+	}
+
+	public get labelWithIcons() {
+		return renderLabelWithIcons(this.label);
+	}
+
 	public get icon() {
-		return icons.testingStatesToIcons.get(this.test.computedState);
+		return icons.testingStatesToIcons.get(this.state);
 	}
 
 	constructor(
 		private readonly results: ITestResult,
 		public readonly test: TestResultItem,
+		public readonly taskIndex: number,
 	) {
 		for (const parent of resultItemParents(results, test)) {
 			if (parent !== test) {
@@ -1331,16 +1336,21 @@ class TestCaseElement implements ITreeElement {
 	}
 }
 
-class TestTaskElement implements ITreeElement {
+class TaskElement implements ITreeElement {
+	public readonly changeEmitter = new Emitter<void>();
+	public readonly onDidChange = this.changeEmitter.event;
 	public readonly type = 'task';
-	public readonly task: ITestRunTask;
 	public readonly context: string;
 	public readonly id: string;
 	public readonly label: string;
-	public readonly icon = undefined;
+	public readonly itemsCache = new CreationCache<TestCaseElement>();
 
-	constructor(results: ITestResult, public readonly test: TestResultItem, index: number) {
-		this.id = `${results.id}/${test.item.extId}/${index}`;
+	public get icon() {
+		return this.results.tasks[this.index].running ? icons.testingStatesToIcons.get(TestResultState.Running) : undefined;
+	}
+
+	constructor(public readonly results: ITestResult, public readonly task: ITestRunTask, public readonly index: number) {
+		this.id = `${results.id}/${index}`;
 		this.task = results.tasks[index];
 		this.context = String(index);
 		this.label = this.task.name ?? localize('testUnnamedTask', 'Unnamed Task');
@@ -1356,6 +1366,7 @@ class TestMessageElement implements ITreeElement {
 	public readonly location?: IRichLocation;
 	public readonly description?: string;
 	public readonly marker?: number;
+	public readonly onDidChange = Event.None;
 
 	constructor(
 		public readonly result: ITestResult,
@@ -1388,7 +1399,7 @@ class TestMessageElement implements ITreeElement {
 	}
 }
 
-type TreeElement = TestResultElement | TestCaseElement | TestMessageElement | TestTaskElement;
+type TreeElement = TestResultElement | TestCaseElement | TestMessageElement | TaskElement;
 
 class OutputPeekTree extends Disposable {
 	private disposed = false;
@@ -1429,6 +1440,15 @@ class OutputPeekTree extends Disposable {
 				compressionEnabled: true,
 				hideTwistiesOfChildlessElements: true,
 				identityProvider: diffIdentityProvider,
+				sorter: {
+					compare(a, b) {
+						if (a instanceof TestCaseElement && b instanceof TestCaseElement) {
+							return cmpPriority(a.state, b.state);
+						}
+
+						return 0;
+					},
+				},
 				accessibilityProvider: {
 					getAriaLabel(element: ITreeElement) {
 						return element.ariaLabel || element.label;
@@ -1440,45 +1460,37 @@ class OutputPeekTree extends Disposable {
 			},
 		)) as WorkbenchCompressibleObjectTree<TreeElement, FuzzyScore>;
 
-		const creationCache = new WeakMap<object, TreeElement>();
-		const cachedCreate = <T extends TreeElement>(ref: object, factory: () => T): TreeElement => {
-			const existing = creationCache.get(ref);
-			if (existing) {
-				return existing;
-			}
+		const cc = new CreationCache<TreeElement>();
+		const getTaskChildren = (taskElem: TaskElement): Iterable<ICompressedTreeElement<TreeElement>> => {
+			const tests = Iterable.filter(taskElem.results.tests, test => test.tasks[taskElem.index].state >= TestResultState.Running || test.tasks[taskElem.index].messages.length > 0);
 
-			const fresh = factory();
-			creationCache.set(ref, fresh);
-			return fresh;
-		};
-
-		const getTaskChildren = (result: ITestResult, test: TestResultItem, taskId: number): Iterable<ICompressedTreeElement<TreeElement>> => {
-			return Iterable.map(test.tasks[0].messages, (m, messageIndex) => ({
-				element: cachedCreate(m, () => new TestMessageElement(result, test, taskId, messageIndex)),
+			return Iterable.map(tests, test => ({
+				element: taskElem.itemsCache.getOrCreate(test, () => new TestCaseElement(taskElem.results, test, taskElem.index)),
 				incompressible: true,
+				children: getTestChildren(taskElem.results, test, taskElem.index),
 			}));
 		};
 
-		const getTestChildren = (result: ITestResult, test: TestResultItem): Iterable<ICompressedTreeElement<TreeElement>> => {
-			const tasks = Iterable.filter(test.tasks, task => task.messages.length > 0);
-			return Iterable.map(tasks, (t, taskId) => ({
-				element: cachedCreate(t, () => new TestTaskElement(result, test, taskId)),
-				incompressible: false,
-				children: getTaskChildren(result, test, taskId),
+		const getTestChildren = (result: ITestResult, test: TestResultItem, taskIndex: number): Iterable<ICompressedTreeElement<TreeElement>> => {
+			return Iterable.map(test.tasks[taskIndex].messages, (m, messageIndex) => ({
+				element: cc.getOrCreate(m, () => new TestMessageElement(result, test, taskIndex, messageIndex)),
+				incompressible: true,
 			}));
 		};
 
 		const getResultChildren = (result: ITestResult): Iterable<ICompressedTreeElement<TreeElement>> => {
-			const tests = Iterable.filter(result.tests, test => test.tasks.some(t => t.messages.length > 0));
-			return Iterable.map(tests, test => ({
-				element: cachedCreate(test, () => new TestCaseElement(result, test)),
-				incompressible: true,
-				children: getTestChildren(result, test),
-			}));
+			return result.tasks.map((task, taskIndex) => {
+				const taskElem = cc.getOrCreate(task, () => new TaskElement(result, task, taskIndex));
+				return ({
+					element: taskElem,
+					incompressible: false,
+					children: getTaskChildren(taskElem),
+				});
+			});
 		};
 
 		const getRootChildren = () => results.results.map(result => {
-			const element = cachedCreate(result, () => new TestResultElement(result));
+			const element = cc.getOrCreate(result, () => new TestResultElement(result));
 			return {
 				element,
 				incompressible: true,
@@ -1489,36 +1501,57 @@ class OutputPeekTree extends Disposable {
 
 		// Queued result updates to prevent spamming CPU when lots of tests are
 		// completing and messaging quickly (#142514)
-		const resultsToUpdate = new Set<ITestResult>();
-		const resultUpdateScheduler = this._register(new RunOnceScheduler(() => {
-			for (const result of resultsToUpdate) {
-				const resultNode = creationCache.get(result);
-				if (resultNode && this.tree.hasElement(resultNode)) {
-					this.tree.setChildren(resultNode, getResultChildren(result), { diffIdentityProvider });
+		const taskChildrenToUpdate = new Set<TaskElement>();
+		const taskChildrenUpdate = this._register(new RunOnceScheduler(() => {
+			for (const taskNode of taskChildrenToUpdate) {
+				if (this.tree.hasElement(taskNode)) {
+					this.tree.setChildren(taskNode, getTaskChildren(taskNode), { diffIdentityProvider });
 				}
 			}
-			resultsToUpdate.clear();
+			taskChildrenToUpdate.clear();
 		}, 300));
 
-		this._register(results.onTestChanged(e => {
-			const itemNode = creationCache.get(e.item);
-			if (itemNode && this.tree.hasElement(itemNode)) { // update to existing test message/state
-				this.tree.setChildren(itemNode, getTestChildren(e.result, e.item));
-				return;
-			}
-
-			const resultNode = creationCache.get(e.result);
-			if (resultNode && this.tree.hasElement(resultNode)) { // new test, update result children
-				if (!resultUpdateScheduler.isScheduled) {
-					resultsToUpdate.add(e.result);
-					resultUpdateScheduler.schedule();
+		const handleNewResults = (result: LiveTestResult) => {
+			const resultNode = cc.get(result)! as TestResultElement;
+			const disposable = new DisposableStore();
+			disposable.add(result.onNewTask(() => {
+				if (this.tree.hasElement(resultNode)) {
+					this.tree.setChildren(resultNode, getResultChildren(result), { diffIdentityProvider });
 				}
-				return;
-			}
+			}));
+			disposable.add(result.onEndTask(index => {
+				(cc.get(result.tasks[index]) as TaskElement | undefined)?.changeEmitter.fire();
+			}));
 
-			// should be unreachable?
-			this.tree.setChildren(null, getRootChildren(), { diffIdentityProvider });
-		}));
+			disposable.add(result.onChange(e => {
+				// try updating the item in each of its tasks
+				for (const [index, task] of result.tasks.entries()) {
+					const taskNode = cc.get(task) as TaskElement;
+					if (!this.tree.hasElement(taskNode)) {
+						continue;
+					}
+
+					const itemNode = taskNode.itemsCache.get(e.item);
+					if (itemNode && this.tree.hasElement(itemNode)) {
+						this.tree.setChildren(itemNode, getTestChildren(result, e.item, index), { diffIdentityProvider });
+						itemNode.changeEmitter.fire();
+						return;
+					}
+
+					taskChildrenToUpdate.add(taskNode);
+					if (!taskChildrenUpdate.isScheduled()) {
+						taskChildrenUpdate.schedule();
+					}
+				}
+			}));
+
+			disposable.add(result.onComplete(() => {
+				resultNode.changeEmitter.fire();
+				disposable.dispose();
+			}));
+
+			this.tree.expand(resultNode, true);
+		};
 
 		this._register(results.onResultsChanged(e => {
 			// little hack here: a result change can cause the peek to be disposed,
@@ -1529,14 +1562,20 @@ class OutputPeekTree extends Disposable {
 			}
 
 			if ('completed' in e) {
-				const resultNode = creationCache.get(e.completed);
-				if (resultNode && this.tree.hasElement(resultNode)) {
-					this.tree.setChildren(resultNode, getResultChildren(e.completed));
-					return;
-				}
+				(cc.get(e.completed) as TestResultElement | undefined)?.changeEmitter.fire();
+				return;
 			}
 
 			this.tree.setChildren(null, getRootChildren(), { diffIdentityProvider });
+
+			// done after setChildren intentionally so that the ResultElement exists in the cache.
+			if ('started' in e) {
+				for (const child of this.tree.getNode(null).children) {
+					this.tree.collapse(child.element, false);
+				}
+
+				handleNewResults(e.started);
+			}
 		}));
 
 		const revealItem = (element: TreeElement, preserveFocus: boolean) => {
@@ -1548,7 +1587,7 @@ class OutputPeekTree extends Disposable {
 		};
 
 		this._register(onDidReveal(({ subject, preserveFocus = false }) => {
-			if (subject instanceof ResultSubject) {
+			if (subject instanceof TaskSubject) {
 				const resultItem = this.tree.getNode(null).children.find(c => (c.element as TestResultElement)?.id === subject.resultId);
 				if (resultItem) {
 					revealItem(resultItem.element as TestResultElement, preserveFocus);
@@ -1556,7 +1595,7 @@ class OutputPeekTree extends Disposable {
 				return;
 			}
 
-			const messageNode = creationCache.get(subject.messages[subject.messageIndex]);
+			const messageNode = cc.get(subject.messages[subject.messageIndex]);
 			if (!messageNode || !this.tree.hasElement(messageNode)) {
 				return;
 			}
@@ -1578,8 +1617,8 @@ class OutputPeekTree extends Disposable {
 		}));
 
 		this._register(this.tree.onDidOpen(async e => {
-			if (e.element instanceof TestResultElement) {
-				this.requestReveal.fire(new ResultSubject(e.element.id));
+			if (e.element instanceof TaskElement) {
+				this.requestReveal.fire(new TaskSubject(e.element.results.id, e.element.index));
 			} else if (e.element instanceof TestMessageElement) {
 				this.requestReveal.fire(new MessageSubject(e.element.result.id, e.element.test, e.element.taskIndex, e.element.messageIndex));
 			}
@@ -1646,7 +1685,7 @@ class TestRunElementRenderer implements ICompressibleTreeRenderer<ITreeElement,
 	public renderCompressedElements(node: ITreeNode<ICompressedTreeNode<ITreeElement>, FuzzyScore>, _index: number, templateData: TemplateData): void {
 		const chain = node.element.elements;
 		const lastElement = chain[chain.length - 1];
-		if (lastElement instanceof TestTaskElement && chain.length >= 2) {
+		if (lastElement instanceof TaskElement && chain.length >= 2) {
 			this.doRender(chain[chain.length - 2], templateData);
 		} else {
 			this.doRender(lastElement, templateData);
@@ -1680,6 +1719,8 @@ class TestRunElementRenderer implements ICompressibleTreeRenderer<ITreeElement,
 
 	/** @inheritdoc */
 	public renderElement(element: ITreeNode<ITreeElement, FuzzyScore>, _index: number, templateData: TemplateData): void {
+		templateData.elementDisposable.clear();
+		templateData.elementDisposable.add(element.element.onDidChange(() => this.doRender(element.element, templateData)));
 		this.doRender(element.element, templateData);
 	}
 
@@ -1689,7 +1730,6 @@ class TestRunElementRenderer implements ICompressibleTreeRenderer<ITreeElement,
 	}
 
 	private doRender(element: ITreeElement, templateData: TemplateData) {
-		templateData.elementDisposable.clear();
 		if (element.labelWithIcons) {
 			dom.reset(templateData.label, ...element.labelWithIcons);
 		} else if (element.description) {
@@ -1733,14 +1773,27 @@ class TreeActionsProvider {
 			const primary: IAction[] = [];
 			const secondary: IAction[] = [];
 
-			if (element instanceof TestResultElement) {
+			if (element instanceof TaskElement) {
 				primary.push(new Action(
 					'testing.outputPeek.showResultOutput',
 					localize('testing.showResultOutput', "Show Result Output"),
 					ThemeIcon.asClassName(Codicon.terminal),
 					undefined,
-					() => this.testTerminalService.open(element.value)
+					() => this.testTerminalService.open(element.results, element.index)
 				));
+			}
+
+			if (element instanceof TestResultElement) {
+				// only show if there are no collapsed test nodes that have more specific choices
+				if (element.value.tasks.length === 1) {
+					primary.push(new Action(
+						'testing.outputPeek.showResultOutput',
+						localize('testing.showResultOutput', "Show Result Output"),
+						ThemeIcon.asClassName(Codicon.terminal),
+						undefined,
+						() => this.testTerminalService.open(element.value, 0)
+					));
+				}
 
 				primary.push(new Action(
 					'testing.outputPeek.reRunLastRun',
@@ -1761,7 +1814,7 @@ class TreeActionsProvider {
 				}
 			}
 
-			if (element instanceof TestCaseElement || element instanceof TestTaskElement) {
+			if (element instanceof TestCaseElement) {
 				const extId = element.test.item.extId;
 				primary.push(new Action(
 					'testing.outputPeek.goToFile',
@@ -1822,7 +1875,7 @@ class TreeActionsProvider {
 						localize('testing.showMessageInTerminal', "Show Output in Terminal"),
 						ThemeIcon.asClassName(Codicon.terminal),
 						undefined,
-						() => this.testTerminalService.open(element.result, element.marker),
+						() => this.testTerminalService.open(element.result, element.taskIndex, element.marker),
 					));
 				}
 			}
@@ -1984,3 +2037,22 @@ export class ToggleTestingPeekHistory extends Action2 {
 		opener.historyVisible.value = !opener.historyVisible.value;
 	}
 }
+
+class CreationCache<T> {
+	private readonly v = new WeakMap<object, T>();
+
+	public get(key: object) {
+		return this.v.get(key);
+	}
+
+	public getOrCreate<T2 extends T>(ref: object, factory: () => T2): T2 {
+		const existing = this.v.get(ref);
+		if (existing) {
+			return existing as T2;
+		}
+
+		const fresh = factory();
+		this.v.set(ref, fresh);
+		return fresh;
+	}
+}
