@@ -92,6 +92,21 @@ export class TestId {
 		return idx === -1 ? undefined : idString.slice(0, idx);
 	}
 
+	/**
+	 * Cheaply gets the local ID of a test identified with the string.
+	 */
+	public static localId(idString: string) {
+		const idx = idString.lastIndexOf(TestIdPathParts.Delimiter);
+		return idx === -1 ? idString : idString.slice(idx + TestIdPathParts.Delimiter.length);
+	}
+
+	/**
+	 * Gets whether maybeChild is a child of maybeParent.
+	 */
+	public static isChild(maybeParent: string, maybeChild: string) {
+		return maybeChild.startsWith(maybeParent) && maybeChild[maybeParent.length] === TestIdPathParts.Delimiter;
+	}
+
 	/**
 	 * Compares the position of the two ID strings.
 	 */
@@ -100,11 +115,11 @@ export class TestId {
 			return TestPosition.IsSame;
 		}
 
-		if (b.startsWith(a + TestIdPathParts.Delimiter)) {
+		if (TestId.isChild(a, b)) {
 			return TestPosition.IsChild;
 		}
 
-		if (a.startsWith(b + TestIdPathParts.Delimiter)) {
+		if (TestId.isChild(b, a)) {
 			return TestPosition.IsParent;
 		}
 
