@@ -6,4 +6,4 @@
 import { registerMainProcessRemoteService } from 'vs/platform/ipc/electron-sandbox/services';
 import { IEncryptionService } from 'vs/workbench/services/encryption/common/encryptionService';
 
-registerMainProcessRemoteService(IEncryptionService, 'encryption', { supportsDelayedInstantiation: true });
+registerMainProcessRemoteService(IEncryptionService, 'encryption');
