@@ -9,6 +9,7 @@ import { DisposableStore } from 'vs/base/common/lifecycle';
 import { mock } from 'vs/base/test/common/mock';
 import { IConfigurationService } from 'vs/platform/configuration/common/configuration';
 import { TestConfigurationService } from 'vs/platform/configuration/test/common/testConfigurationService';
+import { TestDialogService } from 'vs/platform/dialogs/test/common/testDialogService';
 import { ExtensionKind, IEnvironmentService } from 'vs/platform/environment/common/environment';
 import { ExtensionIdentifier, IExtension, IRelaxedExtensionDescription } from 'vs/platform/extensions/common/extensions';
 import { IFileService } from 'vs/platform/files/common/files';
@@ -19,6 +20,8 @@ import { INotificationService } from 'vs/platform/notification/common/notificati
 import { TestNotificationService } from 'vs/platform/notification/test/common/testNotificationService';
 import product from 'vs/platform/product/common/product';
 import { IProductService } from 'vs/platform/product/common/productService';
+import { RemoteAuthorityResolverService } from 'vs/platform/remote/browser/remoteAuthorityResolverService';
+import { IRemoteAuthorityResolverService, ResolverResult } from 'vs/platform/remote/common/remoteAuthorityResolver';
 import { IRemoteExtensionsScannerService } from 'vs/platform/remote/common/remoteExtensionsScanner';
 import { ITelemetryService } from 'vs/platform/telemetry/common/telemetry';
 import { NullTelemetryService } from 'vs/platform/telemetry/common/telemetryUtils';
@@ -145,6 +148,7 @@ suite('ExtensionService', () => {
 			@IRemoteAgentService remoteAgentService: IRemoteAgentService,
 			@IRemoteExtensionsScannerService remoteExtensionsScannerService: IRemoteExtensionsScannerService,
 			@ILifecycleService lifecycleService: ILifecycleService,
+			@IRemoteAuthorityResolverService remoteAuthorityResolverService: IRemoteAuthorityResolverService,
 		) {
 			const extensionsProposedApi = instantiationService.createInstance(ExtensionsProposedApi);
 			const extensionHostFactory = new class implements IExtensionHostFactory {
@@ -172,7 +176,9 @@ suite('ExtensionService', () => {
 				logService,
 				remoteAgentService,
 				remoteExtensionsScannerService,
-				lifecycleService
+				lifecycleService,
+				remoteAuthorityResolverService,
+				new TestDialogService()
 			);
 		}
 
@@ -205,6 +211,9 @@ suite('ExtensionService', () => {
 		protected _onExtensionHostExit(code: number): void {
 			throw new Error('Method not implemented.');
 		}
+		protected _resolveAuthority(remoteAuthority: string): Promise<ResolverResult> {
+			throw new Error('Method not implemented.');
+		}
 	}
 
 	let disposables: DisposableStore;
@@ -236,6 +245,7 @@ suite('ExtensionService', () => {
 			[IUserDataProfileService, TestUserDataProfileService],
 			[IUriIdentityService, UriIdentityService],
 			[IRemoteExtensionsScannerService, TestRemoteExtensionsScannerService],
+			[IRemoteAuthorityResolverService, RemoteAuthorityResolverService]
 		]);
 		extService = <MyTestExtensionService>instantiationService.get(IExtensionService);
 	});
@@ -246,7 +256,34 @@ suite('ExtensionService', () => {
 
 	test('issue #152204: Remote extension host not disposed after closing vscode client', async () => {
 		await extService.startExtensionHosts();
-		extService.stopExtensionHosts();
+		extService.stopExtensionHosts(true);
 		assert.deepStrictEqual(extService.order, (['create 1', 'create 2', 'create 3', 'dispose 3', 'dispose 2', 'dispose 1']));
 	});
+
+	test('Extension host disposed when awaited', async () => {
+		await extService.startExtensionHosts();
+		await extService.stopExtensionHosts(`foo`);
+		assert.deepStrictEqual(extService.order, (['create 1', 'create 2', 'create 3', 'dispose 3', 'dispose 2', 'dispose 1']));
+	});
+
+	test('Extension host not disposed when vetoed (sync)', async () => {
+		await extService.startExtensionHosts();
+
+		extService.onWillStop(e => e.veto(true, 'test 1'));
+		extService.onWillStop(e => e.veto(false, 'test 2'));
+
+		await extService.stopExtensionHosts(`foo`);
+		assert.deepStrictEqual(extService.order, (['create 1', 'create 2', 'create 3']));
+	});
+
+	test('Extension host not disposed when vetoed (async)', async () => {
+		await extService.startExtensionHosts();
+
+		extService.onWillStop(e => e.veto(false, 'test 1'));
+		extService.onWillStop(e => e.veto(Promise.resolve(true), 'test 2'));
+		extService.onWillStop(e => e.veto(Promise.resolve(false), 'test 3'));
+
+		await extService.stopExtensionHosts(`foo`);
+		assert.deepStrictEqual(extService.order, (['create 1', 'create 2', 'create 3']));
+	});
 });
