@@ -456,16 +456,12 @@ export class BrowserHostService extends Disposable implements IHostService {
 
 		const opened = await this.workspaceProvider.open(workspace, options);
 		if (!opened) {
-			const showResult = await this.dialogService.show(
-				Severity.Warning,
-				localize('unableToOpenExternal', "The browser interrupted the opening of a new tab or window. Press 'Open' to open it anyway."),
-				[
-					localize({ key: 'open', comment: ['&& denotes a mnemonic'] }, "&&Open"),
-					localize('cancel', "Cancel")
-				],
-				{ cancelId: 1 }
-			);
-			if (showResult.choice === 0) {
+			const { confirmed } = await this.dialogService.confirm({
+				type: Severity.Warning,
+				message: localize('unableToOpenExternal', "The browser interrupted the opening of a new tab or window. Press 'Open' to open it anyway."),
+				primaryButton: localize({ key: 'open', comment: ['&& denotes a mnemonic'] }, "&&Open")
+			});
+			if (confirmed) {
 				await this.workspaceProvider.open(workspace, options);
 			}
 		}
