@@ -58,6 +58,13 @@ export interface IHoverOptions {
 	 */
 	target: IHoverTarget | HTMLElement;
 
+	/**
+	 * An ID to associate with the hover to be used as an equality check. Normally when calling
+	 * {@link IHoverService.showHover} the options object itself is used to determine if the hover
+	 * is the same one that is already showing, when this is set, the ID will be used instead.
+	 */
+	id?: number | string;
+
 	/**
 	 * A set of actions for the hover's "status bar".
 	 */
@@ -86,6 +93,14 @@ export interface IHoverOptions {
 	 */
 	hideOnHover?: boolean;
 
+	/**
+	 * When {@link hideOnHover} is explicitly true or undefined and its auto value is detected to
+	 * hide, show a hint at the bottom of the hover explaining how to mouse over the widget. This
+	 * should be used in the cases where despite the hover having no interactive content, it's
+	 * likely the user may want to interact with it somehow.
+	 */
+	showHoverHint?: boolean;
+
 	/**
 	 * Whether to hide the hover when a key is pressed.
 	 */
@@ -126,6 +141,14 @@ export interface IHoverOptions {
 	 * - If there are elements in the hover to focus, focus stays inside of the hover when tabbing
 	 */
 	trapFocus?: boolean;
+
+	/*
+	 * The container to pass to {@link IContextViewProvider.showContextView} which renders the hover
+	 * in. This is particularly useful for more natural tab focusing behavior, where the hover is
+	 * created as the next tab index after the element being hovered and/or to workaround the
+	 * element's container hiding on `focusout`.
+	 */
+	container?: HTMLElement;
 }
 
 export interface IHoverAction {
