@@ -6,8 +6,8 @@
 import * as assert from 'assert';
 import { joinPath } from 'vs/base/common/resources';
 import { URI } from 'vs/base/common/uri';
-import { fixRegexNewline, IRgMatch, IRgMessage, RipgrepParser, unicodeEscapesToPCRE2, fixNewline } from 'vs/workbench/services/search/node/ripgrepTextSearchEngine';
-import { Range, TextSearchResult } from 'vs/workbench/services/search/common/searchExtTypes';
+import { fixRegexNewline, IRgMatch, IRgMessage, RipgrepParser, unicodeEscapesToPCRE2, fixNewline, getRgArgs, performBraceExpansionForRipgrep } from 'vs/workbench/services/search/node/ripgrepTextSearchEngine';
+import { Range, TextSearchOptions, TextSearchQuery, TextSearchResult } from 'vs/workbench/services/search/common/searchExtTypes';
 
 suite('RipgrepTextSearchEngine', () => {
 	test('unicodeEscapesToPCRE2', async () => {
@@ -296,4 +296,76 @@ suite('RipgrepTextSearchEngine', () => {
 				]);
 		});
 	});
+
+	suite('getRgArgs', () => {
+		test('simple includes', () => {
+			// Only testing the args that come from includes.
+			function testGetRgArgs(includes: string[], expectedFromIncludes: string[]): void {
+				const query: TextSearchQuery = {
+					pattern: 'test'
+				};
+
+				const options: TextSearchOptions = {
+					includes: includes,
+					excludes: [],
+					maxResults: 1000,
+					useIgnoreFiles: false,
+					followSymlinks: false,
+					useGlobalIgnoreFiles: false,
+					useParentIgnoreFiles: false,
+					folder: URI.file('/some/folder')
+				};
+				const expected = [
+					'--hidden',
+					'--ignore-case',
+					...expectedFromIncludes,
+					'--no-ignore',
+					'--crlf',
+					'--fixed-strings',
+					'--no-config',
+					'--no-ignore-global',
+					'--json',
+					'--',
+					'test',
+					'.'];
+				const result = getRgArgs(query, options);
+				assert.deepStrictEqual(result, expected);
+			}
+
+			([
+				[['a/*', 'b/*'], ['-g', '!*', '-g', '/a', '-g', '/a/*', '-g', '/b', '-g', '/b/*']],
+				[['**/a/*', 'b/*'], ['-g', '!*', '-g', '/b', '-g', '/b/*', '-g', '**/a/*']],
+				[['**/a/*', '**/b/*'], ['-g', '**/a/*', '-g', '**/b/*']],
+				[['foo/*bar/something/**'], ['-g', '!*', '-g', '/foo', '-g', '/foo/*bar', '-g', '/foo/*bar/something', '-g', '/foo/*bar/something/**']],
+			].forEach(([includes, expectedFromIncludes]) => testGetRgArgs(<string[]>includes, <string[]>expectedFromIncludes)));
+		});
+	});
+
+	test('brace expansion for ripgrep', () => {
+		function testBraceExpansion(argGlob: string, expectedGlob: string[]): void {
+			const result = performBraceExpansionForRipgrep(argGlob);
+			assert.deepStrictEqual(result, expectedGlob);
+		}
+
+		[
+			['eep/{a,b}/test', ['eep/a/test', 'eep/b/test']],
+			['eep/{a,b}/{c,d,e}', ['eep/a/c', 'eep/a/d', 'eep/a/e', 'eep/b/c', 'eep/b/d', 'eep/b/e']],
+			['eep/{a,b}/\\{c,d,e}', ['eep/a/{c,d,e}', 'eep/b/{c,d,e}']],
+			['eep/{a,b\\}/test', ['eep/{a,b}/test']],
+			['eep/{a,b\\\\}/test', ['eep/a/test', 'eep/b\\\\/test']],
+			['eep/{a,b\\\\\\}/test', ['eep/{a,b\\\\}/test']],
+			['e\\{ep/{a,b}/test', ['e{ep/a/test', 'e{ep/b/test']],
+			['eep/{a,\\b}/test', ['eep/a/test', 'eep/\\b/test']],
+			['{a/*.*,b/*.*}', ['a/*.*', 'b/*.*']],
+			['{{}', ['{{}']],
+			['aa{{}', ['aa{{}']],
+			['{b{}', ['{b{}']],
+			['{{}c', ['{{}c']],
+			['{{}}', ['{{}}']],
+			['\\{{}}', ['{}']],
+			['{}foo', ['foo']],
+			['bar{ }foo', ['bar foo']],
+			['{}', ['']],
+		].forEach(([includePattern, expectedPatterns]) => testBraceExpansion(<string>includePattern, <string[]>expectedPatterns));
+	});
 });
