@@ -8,7 +8,7 @@ import { IContextKey, RawContextKey, IContextKeyService, ContextKeyExpr } from '
 import { IStorageService, StorageScope, IStorageValueChangeEvent, StorageTarget } from 'vs/platform/storage/common/storage';
 import { IExtensionService } from 'vs/workbench/services/extensions/common/extensions';
 import { Registry } from 'vs/platform/registry/common/platform';
-import { toDisposable, DisposableStore, Disposable, IDisposable } from 'vs/base/common/lifecycle';
+import { toDisposable, DisposableStore, Disposable, IDisposable, DisposableMap } from 'vs/base/common/lifecycle';
 import { ViewPaneContainer, ViewPaneContainerAction, ViewsSubMenu } from 'vs/workbench/browser/parts/views/viewPaneContainer';
 import { SyncDescriptor } from 'vs/platform/instantiation/common/descriptors';
 import { InstantiationType, registerSingleton } from 'vs/platform/instantiation/common/extensions';
@@ -24,6 +24,7 @@ import { IStringDictionary } from 'vs/base/common/collections';
 interface IViewsCustomizations {
 	viewContainerLocations: IStringDictionary<ViewContainerLocation>;
 	viewLocations: IStringDictionary<string>;
+	viewContainerBadgeEnablementStates: IStringDictionary<boolean>;
 }
 
 function getViewContainerStorageId(viewContainerId: string): string { return `${viewContainerId}.state`; }
@@ -44,8 +45,8 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 	private readonly _onDidChangeContainerLocation: Emitter<{ viewContainer: ViewContainer; from: ViewContainerLocation; to: ViewContainerLocation }> = this._register(new Emitter<{ viewContainer: ViewContainer; from: ViewContainerLocation; to: ViewContainerLocation }>());
 	readonly onDidChangeContainerLocation: Event<{ viewContainer: ViewContainer; from: ViewContainerLocation; to: ViewContainerLocation }> = this._onDidChangeContainerLocation.event;
 
-	private readonly viewContainerModels: Map<ViewContainer, { viewContainerModel: ViewContainerModel; disposable: IDisposable }>;
-	private readonly viewsVisibilityActionDisposables: Map<ViewContainer, DisposableStore>;
+	private readonly viewContainerModels = this._register(new DisposableMap<ViewContainer, { viewContainerModel: ViewContainerModel; disposables: DisposableStore } & IDisposable>());
+	private readonly viewsVisibilityActionDisposables = this._register(new DisposableMap<ViewContainer, IDisposable>());
 	private readonly activeViewContextKeys: Map<string, IContextKey<boolean>>;
 	private readonly movableViewContextKeys: Map<string, IContextKey<boolean>>;
 	private readonly defaultViewLocationContextKeys: Map<string, IContextKey<boolean>>;
@@ -56,6 +57,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 
 	private viewContainersCustomLocations: Map<string, ViewContainerLocation>;
 	private viewDescriptorsCustomLocations: Map<string, string>;
+	private viewContainerBadgeEnablementStates: Map<string, boolean>;
 
 	private readonly _onDidChangeViewContainers = this._register(new Emitter<{ added: ReadonlyArray<{ container: ViewContainer; location: ViewContainerLocation }>; removed: ReadonlyArray<{ container: ViewContainer; location: ViewContainerLocation }> }>());
 	readonly onDidChangeViewContainers = this._onDidChangeViewContainers.event;
@@ -70,8 +72,6 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 	) {
 		super();
 
-		this.viewContainerModels = new Map<ViewContainer, { viewContainerModel: ViewContainerModel; disposable: IDisposable }>();
-		this.viewsVisibilityActionDisposables = new Map<ViewContainer, DisposableStore>();
 		this.activeViewContextKeys = new Map<string, IContextKey<boolean>>();
 		this.movableViewContextKeys = new Map<string, IContextKey<boolean>>();
 		this.defaultViewLocationContextKeys = new Map<string, IContextKey<boolean>>();
@@ -83,6 +83,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 		this.migrateToViewsCustomizationsStorage();
 		this.viewContainersCustomLocations = new Map<string, ViewContainerLocation>(Object.entries(this.viewCustomizations.viewContainerLocations));
 		this.viewDescriptorsCustomLocations = new Map<string, string>(Object.entries(this.viewCustomizations.viewLocations));
+		this.viewContainerBadgeEnablementStates = new Map<string, boolean>(Object.entries(this.viewCustomizations.viewContainerBadgeEnablementStates));
 
 		// Register all containers that were registered before this ctor
 		this.viewContainers.forEach(viewContainer => this.onDidRegisterViewContainer(viewContainer));
@@ -102,16 +103,9 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 			this._onDidChangeViewContainers.fire({ removed: [{ container: viewContainer, location: this.getViewContainerLocation(viewContainer) }], added: [] });
 		}));
 
-		this._register(toDisposable(() => {
-			this.viewContainerModels.forEach(({ disposable }) => disposable.dispose());
-			this.viewContainerModels.clear();
-			this.viewsVisibilityActionDisposables.forEach(disposables => disposables.dispose());
-			this.viewsVisibilityActionDisposables.clear();
-		}));
-
 		this._register(this.storageService.onDidChangeValue((e) => { this.onDidStorageChange(e); }));
 
-		this._register(this.extensionService.onDidRegisterExtensions(() => this.onDidRegisterExtensions()));
+		this.extensionService.whenInstalledExtensionsRegistered().then(() => this.whenExtensionsRegistered());
 
 	}
 
@@ -131,6 +125,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 		const viewsCustomizations: IViewsCustomizations = {
 			viewContainerLocations: viewContainerLocations.reduce<IStringDictionary<ViewContainerLocation>>((result, [id, location]) => { result[id] = location; return result; }, {}),
 			viewLocations: viewDescriptorLocations.reduce<IStringDictionary<string>>((result, [id, { containerId }]) => { result[id] = containerId; return result; }, {}),
+			viewContainerBadgeEnablementStates: {}
 		};
 		this.storageService.store(ViewDescriptorService.VIEWS_CUSTOMIZATIONS, JSON.stringify(viewsCustomizations), StorageScope.PROFILE, StorageTarget.USER);
 		this.storageService.remove('views.cachedViewContainerLocations', StorageScope.PROFILE);
@@ -191,7 +186,8 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 		}
 	}
 
-	onDidRegisterExtensions(): void {
+	whenExtensionsRegistered(): void {
+
 		// Handle those views whose custom parent view container does not exist anymore
 		// May be the extension contributing this view container is no longer installed
 		// Or the parent view container is generated and no longer available.
@@ -204,6 +200,9 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 
 		// Save updated view customizations after cleanup
 		this.saveViewCustomizations();
+
+		// Register visibility actions for all views
+		this.registerViewsVisibilityActions();
 	}
 
 	private onDidRegisterViews(views: { views: IViewDescriptor[]; viewContainer: ViewContainer }[]): void {
@@ -304,6 +303,15 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 		this.saveViewCustomizations();
 	}
 
+	getViewContainerBadgeEnablementState(id: string): boolean {
+		return this.viewContainerBadgeEnablementStates.get(id) ?? true;
+	}
+
+	setViewContainerBadgeEnablementState(id: string, badgesEnabled: boolean): void {
+		this.viewContainerBadgeEnablementStates.set(id, badgesEnabled);
+		this.saveViewCustomizations();
+	}
+
 	moveViewToLocation(view: IViewDescriptor, location: ViewContainerLocation): void {
 		const container = this.registerGeneratedViewContainer(location);
 		this.moveViewsToContainer([view], container);
@@ -470,6 +478,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 		}
 
 		this.viewContainersCustomLocations.delete(viewContainerId);
+		this.viewContainerBadgeEnablementStates.delete(viewContainerId);
 
 		// Clean up caches of container
 		this.storageService.remove(getViewsStateStorageId(viewContainer?.storageId || getViewContainerStorageId(viewContainerId)), StorageScope.PROFILE);
@@ -565,7 +574,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 		}
 		// Execute View Movements
 		for (const { views, from, to } of viewsToMove) {
-			this.moveViewsWithoutSaving(views, from, to);
+			this.moveViewsWithoutSaving(views, from, to, ViewVisibilityState.Default);
 		}
 
 		this.viewContainersCustomLocations = newViewContainerCustomizations;
@@ -581,7 +590,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 	}
 
 	private saveViewCustomizations(): void {
-		const viewCustomizations: IViewsCustomizations = { viewContainerLocations: {}, viewLocations: {} };
+		const viewCustomizations: IViewsCustomizations = { viewContainerLocations: {}, viewLocations: {}, viewContainerBadgeEnablementStates: {} };
 
 		for (const [containerId, location] of this.viewContainersCustomLocations) {
 			const container = this.getViewContainerById(containerId);
@@ -605,6 +614,12 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 			viewCustomizations.viewLocations[viewId] = viewContainerId;
 		}
 
+		// Loop through viewContainerBadgeEnablementStates and save only the ones that are disabled
+		for (const [viewContainerId, badgeEnablementState] of this.viewContainerBadgeEnablementStates) {
+			if (badgeEnablementState === false) {
+				viewCustomizations.viewContainerBadgeEnablementStates[viewContainerId] = badgeEnablementState;
+			}
+		}
 		this.viewCustomizations = viewCustomizations;
 	}
 
@@ -614,6 +629,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 			this._viewCustomizations = JSON.parse(this.getStoredViewCustomizationsValue()) as IViewsCustomizations;
 			this._viewCustomizations.viewContainerLocations = this._viewCustomizations.viewContainerLocations ?? {};
 			this._viewCustomizations.viewLocations = this._viewCustomizations.viewLocations ?? {};
+			this._viewCustomizations.viewContainerBadgeEnablementStates = this._viewCustomizations.viewContainerBadgeEnablementStates ?? {};
 		}
 		return this._viewCustomizations;
 	}
@@ -678,21 +694,11 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 			viewContainerModel.onDidAddVisibleViewDescriptors(added => this.onDidChangeVisibleViews({ added: added.map(({ viewDescriptor }) => viewDescriptor), removed: [] }), this, disposables);
 			viewContainerModel.onDidRemoveVisibleViewDescriptors(removed => this.onDidChangeVisibleViews({ added: [], removed: removed.map(({ viewDescriptor }) => viewDescriptor) }), this, disposables);
 
-			this.registerViewsVisibilityActions(viewContainerModel);
-			disposables.add(Event.any(
-				viewContainerModel.onDidChangeActiveViewDescriptors,
-				viewContainerModel.onDidAddVisibleViewDescriptors,
-				viewContainerModel.onDidRemoveVisibleViewDescriptors,
-				viewContainerModel.onDidMoveVisibleViewDescriptors
-			)(e => this.registerViewsVisibilityActions(viewContainerModel!)));
-			disposables.add(toDisposable(() => {
-				this.viewsVisibilityActionDisposables.get(viewContainer)?.dispose();
-				this.viewsVisibilityActionDisposables.delete(viewContainer);
-			}));
+			disposables.add(toDisposable(() => this.viewsVisibilityActionDisposables.deleteAndDispose(viewContainer)));
 
 			disposables.add(this.registerResetViewContainerAction(viewContainer));
 
-			this.viewContainerModels.set(viewContainer, { viewContainerModel: viewContainerModel, disposable: disposables });
+			this.viewContainerModels.set(viewContainer, { viewContainerModel: viewContainerModel, disposables, dispose: () => disposables.dispose() });
 
 			// Register all views that were statically registered to this container
 			// Potentially, this is registering something that was handled by another container
@@ -713,11 +719,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 	}
 
 	private onDidDeregisterViewContainer(viewContainer: ViewContainer): void {
-		const viewContainerModelItem = this.viewContainerModels.get(viewContainer);
-		if (viewContainerModelItem) {
-			viewContainerModelItem.disposable.dispose();
-			this.viewContainerModels.delete(viewContainer);
-		}
+		this.viewContainerModels.deleteAndDispose(viewContainer);
 	}
 
 	private onDidChangeActiveViews({ added, removed }: { added: ReadonlyArray<IViewDescriptor>; removed: ReadonlyArray<IViewDescriptor> }): void {
@@ -734,16 +736,23 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 		});
 	}
 
-	private registerViewsVisibilityActions(viewContainerModel: ViewContainerModel): void {
-		let disposables = this.viewsVisibilityActionDisposables.get(viewContainerModel.viewContainer);
-		if (!disposables) {
-			disposables = new DisposableStore();
-			this.viewsVisibilityActionDisposables.set(viewContainerModel.viewContainer, disposables);
+	private registerViewsVisibilityActions(): void {
+		for (const [viewContainer, { viewContainerModel, disposables }] of this.viewContainerModels) {
+			this.viewsVisibilityActionDisposables.set(viewContainer, this.registerViewsVisibilityActionsForContainer(viewContainerModel));
+			disposables.add(Event.any(
+				viewContainerModel.onDidChangeActiveViewDescriptors,
+				viewContainerModel.onDidAddVisibleViewDescriptors,
+				viewContainerModel.onDidRemoveVisibleViewDescriptors,
+				viewContainerModel.onDidMoveVisibleViewDescriptors
+			)(e => this.viewsVisibilityActionDisposables.set(viewContainer, this.registerViewsVisibilityActionsForContainer(viewContainerModel))));
 		}
-		disposables.clear();
+	}
+
+	private registerViewsVisibilityActionsForContainer(viewContainerModel: ViewContainerModel): IDisposable {
+		const disposables = new DisposableStore();
 		viewContainerModel.activeViewDescriptors.forEach((viewDescriptor, index) => {
 			if (!viewDescriptor.remoteAuthority) {
-				disposables?.add(registerAction2(class extends ViewPaneContainerAction<ViewPaneContainer> {
+				disposables.add(registerAction2(class extends ViewPaneContainerAction<ViewPaneContainer> {
 					constructor() {
 						super({
 							id: `${viewDescriptor.id}.toggleVisibility`,
@@ -780,7 +789,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 						viewPaneContainer.toggleViewVisibility(viewDescriptor.id);
 					}
 				}));
-				disposables?.add(registerAction2(class extends ViewPaneContainerAction<ViewPaneContainer> {
+				disposables.add(registerAction2(class extends ViewPaneContainerAction<ViewPaneContainer> {
 					constructor() {
 						super({
 							id: `${viewDescriptor.id}.removeView`,
@@ -804,6 +813,7 @@ export class ViewDescriptorService extends Disposable implements IViewDescriptor
 				}));
 			}
 		});
+		return disposables;
 	}
 
 	private registerResetViewContainerAction(viewContainer: ViewContainer): IDisposable {
