@@ -4,7 +4,7 @@
  *--------------------------------------------------------------------------------------------*/
 
 import { ViewContainer, IViewsRegistry, IViewDescriptor, Extensions as ViewExtensions, IViewContainerModel, IAddedViewDescriptorRef, IViewDescriptorRef, IAddedViewDescriptorState, defaultViewIcon } from 'vs/workbench/common/views';
-import { IContextKeyService, IReadableSet } from 'vs/platform/contextkey/common/contextkey';
+import { IContextKeyService } from 'vs/platform/contextkey/common/contextkey';
 import { IStorageService, StorageScope, IStorageValueChangeEvent, StorageTarget } from 'vs/platform/storage/common/storage';
 import { Registry } from 'vs/platform/registry/common/platform';
 import { Disposable } from 'vs/base/common/lifecycle';
@@ -14,15 +14,18 @@ import { URI } from 'vs/base/common/uri';
 import { coalesce, move } from 'vs/base/common/arrays';
 import { isUndefined, isUndefinedOrNull } from 'vs/base/common/types';
 import { isEqual, joinPath } from 'vs/base/common/resources';
-import { ThemeIcon } from 'vs/platform/theme/common/themeService';
+import { ThemeIcon } from 'vs/base/common/themables';
 import { IStringDictionary } from 'vs/base/common/collections';
 import { localize } from 'vs/nls';
 import { ILogger, ILoggerService } from 'vs/platform/log/common/log';
 import { IWorkbenchEnvironmentService } from 'vs/workbench/services/environment/common/environmentService';
 import { Action2, registerAction2 } from 'vs/platform/actions/common/actions';
 import { Categories } from 'vs/platform/action/common/actionCommonCategories';
-import { IOutputChannelRegistry, IOutputService, Extensions as OutputExtensions } from 'vs/workbench/services/output/common/output';
+import { IOutputService } from 'vs/workbench/services/output/common/output';
+import { CounterSet } from 'vs/base/common/map';
 
+const VIEWS_LOG_ID = 'viewsLog';
+const VIEWS_LOG_NAME = localize('views log', "Views");
 function getViewsLogFile(environmentService: IWorkbenchEnvironmentService): URI { return joinPath(environmentService.windowLogsPath, 'views.log'); }
 
 registerAction2(class extends Action2 {
@@ -35,48 +38,17 @@ registerAction2(class extends Action2 {
 		});
 	}
 	async run(servicesAccessor: ServicesAccessor): Promise<void> {
+		const loggerService = servicesAccessor.get(ILoggerService);
 		const outputService = servicesAccessor.get(IOutputService);
 		const environmentService = servicesAccessor.get(IWorkbenchEnvironmentService);
-		Registry.as<IOutputChannelRegistry>(OutputExtensions.OutputChannels).registerChannel({ id: 'viewsLog', label: localize('views log', "Views"), file: getViewsLogFile(environmentService), log: true });
-		outputService.showChannel('viewsLog');
+		loggerService.setVisibility(getViewsLogFile(environmentService), true);
+		outputService.showChannel(VIEWS_LOG_ID);
 
 	}
 });
 
 export function getViewsStateStorageId(viewContainerStorageId: string): string { return `${viewContainerStorageId}.hidden`; }
 
-class CounterSet<T> implements IReadableSet<T> {
-
-	private map = new Map<T, number>();
-
-	add(value: T): CounterSet<T> {
-		this.map.set(value, (this.map.get(value) || 0) + 1);
-		return this;
-	}
-
-	delete(value: T): boolean {
-		let counter = this.map.get(value) || 0;
-
-		if (counter === 0) {
-			return false;
-		}
-
-		counter--;
-
-		if (counter === 0) {
-			this.map.delete(value);
-		} else {
-			this.map.set(value, counter);
-		}
-
-		return true;
-	}
-
-	has(value: T): boolean {
-		return this.map.has(value);
-	}
-}
-
 interface IStoredWorkspaceViewState {
 	collapsed: boolean;
 	isHidden: boolean;
@@ -119,7 +91,7 @@ class ViewDescriptorsState extends Disposable {
 	) {
 		super();
 
-		this.logger = loggerService.createLogger(getViewsLogFile(workbenchEnvironmentService));
+		this.logger = loggerService.createLogger(getViewsLogFile(workbenchEnvironmentService), { id: VIEWS_LOG_ID, name: VIEWS_LOG_NAME, hidden: true });
 
 		this.globalViewsStateStorageId = getViewsStateStorageId(viewContainerStorageId);
 		this.workspaceViewsStateStorageId = viewContainerStorageId;
@@ -391,22 +363,12 @@ export class ViewContainerModel extends Disposable implements IViewContainerMode
 	) {
 		super();
 
-		this.logger = loggerService.createLogger(getViewsLogFile(workbenchEnvironmentService));
+		this.logger = loggerService.createLogger(getViewsLogFile(workbenchEnvironmentService), { id: VIEWS_LOG_ID, name: VIEWS_LOG_NAME, hidden: true });
 
 		this._register(Event.filter(contextKeyService.onDidChangeContext, e => e.affectsSome(this.contextKeys))(() => this.onDidChangeContext()));
 		this.viewDescriptorsState = this._register(instantiationService.createInstance(ViewDescriptorsState, viewContainer.storageId || `${viewContainer.id}.state`, typeof viewContainer.title === 'string' ? viewContainer.title : viewContainer.title.original));
 		this._register(this.viewDescriptorsState.onDidChangeStoredState(items => this.updateVisibility(items)));
 
-		this._register(Event.any(
-			Event.map(this.onDidAddVisibleViewDescriptors, added => `Added views:${added.map(v => v.viewDescriptor.id).join(',')} in ${this.viewContainer.id}`),
-			Event.map(this.onDidRemoveVisibleViewDescriptors, removed => `Removed views:${removed.map(v => v.viewDescriptor.id).join(',')} from ${this.viewContainer.id}`),
-			Event.map(this.onDidMoveVisibleViewDescriptors, ({ from, to }) => `Moved view ${from.viewDescriptor.id} to ${to.viewDescriptor.id} in ${this.viewContainer.id}`))
-			(message => {
-				this.logger.info(message);
-				this.viewDescriptorsState.updateState(this.allViewDescriptors);
-				this.updateContainerInfo();
-			}));
-
 		this.updateContainerInfo();
 	}
 
@@ -555,10 +517,7 @@ export class ViewContainerModel extends Disposable implements IViewContainerMode
 			this.viewDescriptorItems[index].state.order = index;
 		}
 
-		this._onDidMoveVisibleViewDescriptors.fire({
-			from: { index: fromIndex, viewDescriptor: fromViewDescriptor.viewDescriptor },
-			to: { index: toIndex, viewDescriptor: toViewDescriptor.viewDescriptor }
-		});
+		this.broadCastMovedViewDescriptors({ index: fromIndex, viewDescriptor: fromViewDescriptor.viewDescriptor }, { index: toIndex, viewDescriptor: toViewDescriptor.viewDescriptor });
 	}
 
 	add(addedViewDescriptorStates: IAddedViewDescriptorState[]): void {
@@ -707,15 +666,28 @@ export class ViewContainerModel extends Disposable implements IViewContainerMode
 	private broadCastAddedVisibleViewDescriptors(added: IAddedViewDescriptorRef[]): void {
 		if (added.length) {
 			this._onDidAddVisibleViewDescriptors.fire(added.sort((a, b) => a.index - b.index));
+			this.updateState(`Added views:${added.map(v => v.viewDescriptor.id).join(',')} in ${this.viewContainer.id}`);
 		}
 	}
 
 	private broadCastRemovedVisibleViewDescriptors(removed: IViewDescriptorRef[]): void {
 		if (removed.length) {
 			this._onDidRemoveVisibleViewDescriptors.fire(removed.sort((a, b) => b.index - a.index));
+			this.updateState(`Removed views:${removed.map(v => v.viewDescriptor.id).join(',')} from ${this.viewContainer.id}`);
 		}
 	}
 
+	private broadCastMovedViewDescriptors(from: IViewDescriptorRef, to: IViewDescriptorRef): void {
+		this._onDidMoveVisibleViewDescriptors.fire({ from, to });
+		this.updateState(`Moved view ${from.viewDescriptor.id} to ${to.viewDescriptor.id} in ${this.viewContainer.id}`);
+	}
+
+	private updateState(reason: string): void {
+		this.logger.info(reason);
+		this.viewDescriptorsState.updateState(this.allViewDescriptors);
+		this.updateContainerInfo();
+	}
+
 	private isViewDescriptorVisible(viewDescriptorItem: IViewDescriptorItem): boolean {
 		if (!viewDescriptorItem.state.active) {
 			return false;
